Package: svyROC
Title: Estimation of the ROC Curve and the AUC for Complex Survey Data
Version: 1.1.0
Authors@R: 
    c(person("Amaia", "Iparragirre", , "amaia.iparragirre@ehu.eus", role = c("aut", "cre", "cph"),
             comment = c(ORCID = "0000-0002-0660-6535")),
      person("Irantzu", "Barrio", role = c("aut")),
      person("Inmaculada", "Arostegui", role = c("aut")))
Maintainer: Amaia Iparragirre <amaia.iparragirre@ehu.eus>
Description: Estimate the receiver operating characteristic (ROC) curve, area under the curve (AUC) and optimal cut-off points for individual classification taking into account complex sampling designs when working with complex survey data. Methods implemented in this package are described in: A. Iparragirre, I. Barrio, I. Arostegui (2024) <doi:10.1002/sta4.635>; A. Iparragirre, I. Barrio, J. Aramendi, I. Arostegui (2022) <doi:10.2436/20.8080.02.121>; A. Iparragirre, I. Barrio (2024) <doi:10.1007/978-3-031-65723-8_7>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
Imports: survey, svyVarSel
NeedsCompilation: no
Packaged: 2026-04-13 09:40:24 UTC; amaiaiparragirre
Author: Amaia Iparragirre [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-0660-6535>),
  Irantzu Barrio [aut],
  Inmaculada Arostegui [aut]
Repository: CRAN
Date/Publication: 2026-04-14 10:20:07 UTC
Built: R 4.6.0; ; 2026-04-26 15:26:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ci.wauc)
export(corrected.wauc)
export(ht.indep)
export(ht.paired)
export(wauc)
export(wocp)
export(wroc)
export(wroc.plot)
export(wse)
export(wsp)
import(survey)
import(svyVarSel)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,quasibinomial)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,weights)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# svyROC

The goal of the svyROC package is to plot weighted estimates of the ROC
curves and to obtain weighted estimates of the AUC.

The following functions are available:

- `wse`, `wsp`: estimate sensitivity and specificity parameters for a
  specific cut-off point considering sampling weights.
- `wroc`: estimate the ROC curve considering sampling weights.
- `wauc`: estimate the AUC considering sampling weights.
- `corrected.wauc`: correct the optimism of the weighted estimate of the
  AUC by means of replicate weights.
- `wocp`: calculate optimal cut-off points for individual classification
  considering sampling weights.
- `wroc.plot`: plot the ROC curve.
- `ci.wauc`: confidence intervals for the AUC.
- `ht.indep`: hypothesis test for the comparison of two independent
  AUCs.
- `ht.paired`: hypothesis test for the comparison of two paired AUCs.

The methodology proposed for the above-mentioned functions can be found
in the following **references**:

- Iparragirre, A., Barrio, I., Aramendi, J. and Arostegui, I. (2022).
  Estimation of cut-off points under complex-sampling design data.
  *SORT-Statistics and Operations Research Transactions* **46**(1),
  137–158.

- Iparragirre, A., Barrio, I. and Arostegui, I. (2023). Estimation of
  the ROC curve and the area under it with complex survey data. *Stat*
  **12**(1), e635.

- Iparragirre, A. and Barrio, I. (2024). Optimism Correction of the AUC
  with Complex Survey Data. In: Einbeck, J., Maeng, H., Ogundimu, E.,
  Perrakis, K. (eds) Developments in Statistical Modelling. IWSM 2024.
  Contributions to Statistics. Springer, Cham.

## Installation

To install the package from CRAN:

``` r
install.packages("svyROC")
```

To install the most updated version of the package from GitHub run the
following code:

``` r
devtools::install_github("aiparragirre/svyROC")
```

## Example

We need information on three elements for each unit in the sample in
order to estimate the ROC curve (`wroc()` function) and AUC (`wauc()`
function):

- `response.var`: variable indicating the dichotomous response variable.
- `phat.var`: predicted probabilities of event.
- `weights.var`: variable indicating the sampling weights.

We can put these three vectors in a data frame, or save them separately
in three different vectors. The data set `example_data_wroc` is set as
an example in the package. We also need to define the tags for events
and non-events.

``` r
library(svyROC)
data(example_data_wroc)

mycurve <- wroc(response.var = "y", phat.var = "phat", weights.var = "weights",
                data = example_data_wroc,
                tag.event = 1, tag.nonevent = 0)

# Or equivalently
mycurve <- wroc(response.var = example_data_wroc$y,
                phat.var = example_data_wroc$phat,
                weights.var = example_data_wroc$weights,
                tag.event = 1, tag.nonevent = 0)
```

Similarly, we can run the following code to estimate the AUC:

``` r
auc.obj <- wauc(response.var = "y",
                phat.var = "phat",
                weights.var = "weights",
                tag.event = 1,
                tag.nonevent = 0,
                data = example_data_wroc)

# Or equivalently
auc.obj <- wauc(response.var = example_data_wroc$y,
                phat.var = example_data_wroc$phat,
                weights.var = example_data_wroc$weights,
                tag.event = 1, tag.nonevent = 0)
```

We can correct the optimism of the weighted estimate of the AUC by means
of replicate weights, as proposed in Iparragirre and Barrio (2024), by
means of the `corrected.wauc()` function. For this purpose, we
additionally need information on the covariates and the sampling design.
Here is an example of the usage of this function:

``` r

data(example_variables_wroc)
mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata,
                              weights = ~weights, nest = TRUE,
                              data = example_variables_wroc)
m <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = mydesign,
                    family = quasibinomial())
phat <- predict(m, newdata = example_variables_wroc, type = "response")
myaucw <- wauc(response.var = example_variables_wroc$y, phat.var = phat,
               weights.var = example_variables_wroc$weights)

# Correction of the AUCw:
set.seed(1)
cor <- corrected.wauc(data = example_variables_wroc,
                      formula = y ~ x1 + x2 + x3 + x4 + x5 + x6,
                      tag.event = 1, tag.nonevent = 0,
                      weights.var = "weights", strata.var = "strata", cluster.var = "cluster",
                      method = "dCV", dCV.method = "pooling", k = 10, R = 20)
# Or equivalently:
set.seed(1)
cor <- corrected.wauc(design = mydesign,
                      formula = y ~ x1 + x2 + x3 + x4 + x5 + x6,
                      tag.event = 1, tag.nonevent = 0,
                      method = "dCV", dCV.method = "pooling", k = 10, R = 20)
```

We can also estimate the sensitivity (`wse()`) and specificity (`wsp()`)
parameters for a specific cut-off point considering sampling weights.
For this purpose, we need to indicate the cut-off point we want to use
in the function by means of the argument `cutoff.value`:

``` r

# Specificity ----------------------------------------------------------

sp.obj <- wsp(response.var = "y",
              phat.var = "phat",
              weights.var = "weights",
              tag.nonevent = 0,
              cutoff.value = 0.5,
              data = example_data_wroc)

# Or equivalently
sp.obj <- wsp(response.var = example_data_wroc$y,
              phat.var = example_data_wroc$phat,
              weights.var = example_data_wroc$weights,
              tag.nonevent = 0,
              cutoff.value = 0.5)
   
# Sensitivity ----------------------------------------------------------

se.obj <- wse(response.var = "y",
              phat.var = "phat",
              weights.var = "weights",
              tag.event = 1,
              cutoff.value = 0.5,
              data = example_data_wroc)

# Or equivalently
se.obj <- wse(response.var = example_data_wroc$y,
              phat.var = example_data_wroc$phat,
              weights.var = example_data_wroc$weights,
              tag.event = 1,
              cutoff.value = 0.5)
```

Finally, use the function `wocp()` to obtain optimal cut-off points for
individual classification as proposed in Iparragirre et al (2022). Some
functions of the package `OptimalCutpoints` have been modified in order
for them to consider sampling weights:

Lopez-Raton, M., Rodriguez-Alvarez, M.X, Cadarso-Suarez, C. and
Gude-Sampedro, F. (2014). OptimalCutpoints: An R Package for Selecting
Optimal Cutpoints in Diagnostic Tests. *Journal of Statistical Software*
**61**(8), 1–36.

One of the methods proposed in the paper needs to be selected when
running the function by means of the argument `method`: `Youden`,
`MaxProdSpSe`, `ROC01` or `MaxEfficiency`.

``` r
myocp <- wocp(response.var = "y",
              phat.var = "phat", weights.var = "weights",
              tag.event = 1,
              tag.nonevent = 0,
              method = "Youden",
              data = example_data_wroc)

# Or equivalently
myocp <- wocp(example_data_wroc$y,
              example_data_wroc$phat,
              example_data_wroc$weights,
              tag.event = 1,
              tag.nonevent = 0,
              method = "Youden")
```

If you want to draw the optimal cut-off point in the ROC curve, then use
the function `wroc.plot()` and indicate the method by means of the
argument `cutoff.method` in the function `wroc()` as follows:

``` r
mycurve <- wroc(response.var = "y",
                phat.var = "phat",
                weights.var = "weights",
                data = example_data_wroc,
                tag.event = 1,
                tag.nonevent = 0,
                cutoff.method = "Youden")
wroc.plot(x = mycurve,
          print.auc = TRUE,
          print.cutoff = TRUE)
```

<img src="man/figures/README-unnamed-chunk-9-1.png" width="100%" />

#' Optimal cut-off points for complex survey data
#'
#' Calculate optimal cut-off points for complex survey data (Iparragirre et al., 2022).
#' Some functions of the package OptimalCutpoints (Lopez-Raton et al, 2014) have been used and modified in order them to consider sampling weights.
#'
#' @param response.var A character string with the name of the column indicating the response variable in the data set
#' or a vector (either numeric or character string) with information of the response variable for all the units.
#' @param phat.var A character string with the name of the column indicating the estimated probabilities in the data set
#' or a numeric vector containing estimated probabilities for all the units.
#' @param weights.var A character string indicating the name of the column with sampling weights or
#' a numeric vector containing information of the sampling weights.
#' It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
#' For unweighted estimates, set all the sampling weight values to 1.
#' @param tag.event A character string indicating the label used to indicate the event of interest in \code{response.var}.
#' The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.
#' @param tag.nonevent A character string indicating the label used for non-event in \code{response.var}.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param method A character string indicating the method to be used to select the optimal cut-off point.
#' Choose one of the following methods (Lopez-Raton et al, 2014):
#' `MaxProdSpSe`, `ROC01`, `Youden`, `MaxEfficiency`.
#' @param data A data frame which, at least, must incorporate information on the columns
#' \code{response.var}, \code{phat.var} and \code{weights.var}.
#' If \code{data=NULL}, then specific numerical vectors must be included in
#' \code{response.var}, \code{phat.var} and \code{weights.var},
#' or the sampling design should be indicated in the argument \code{design}.
#' @param design An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data.
#' If \code{design = NULL}, information on the data set (argument \code{data})
#' and/or sampling weights (argument \code{weights.var}) must be included.
#'
#' @details
#' Let \eqn{S} indicate a sample of \eqn{n} observations of the vector of random variables \eqn{(Y,\pmb X)}, and \eqn{\forall i=1,\ldots,n,} \eqn{y_i} indicate the \eqn{i^{th}} observation of the response variable \eqn{Y},
#' and \eqn{\pmb x_i} the observations of the vector covariates \eqn{\pmb X}. Let \eqn{w_i} indicate the sampling weight corresponding to the unit \eqn{i} and \eqn{\hat p_i} the estimated probability of event.
#' Let \eqn{S_0} and \eqn{S_1} be subsamples of \eqn{S}, formed by the units without the event of interest (\eqn{y_i=0}) and with the event of interest (\eqn{y_i=1}), respectively.
#' Then, the optimal cut-off points are obtained as follows:
#' - `Youden`: \deqn{c_w^{\text{Youden}}=argmax_c\{\widehat{Se}_w(c) + \widehat{Sp}_w(c)-1\},}
#' - `MaxProdSpSe`: \deqn{c_w^{\text{MaxProdSpSe}}=argmax_c\{\widehat{Se}_w(c) * \widehat{Sp}_w(c)\},}
#' - `ROC01`: \deqn{c_w^{\text{ROC01}}=argmax_c\{(\widehat{Se}_w(c)-1)^2 + (\widehat{Sp}_w(c)-1)^2\},}
#' - `MaxEfficiency`: \deqn{c_w^{\text{MaxEfficiency}}=argmax_c\{\hat p_{Y,w}\widehat{Se}_w(c) + (1-\hat p_{Y,w})\widehat{Sp}_w(c)\},}
#'
#' where, the sensitivity and specificity parameters for a given cut-off point \eqn{c} are estimated as follows:
#' \deqn{\widehat{Se}_w(c)=\dfrac{\sum_{i\in S_1}w_i\cdot I (\hat p_i\geq c)}{\sum_{i\in S_1}w_i}\:;\:\widehat{Sp}_w(c)=\dfrac{\sum_{i\in S_0}w_i\cdot I (\hat p_i<c)}{\sum_{i\in S_0}w_i},}
#' and,
#' \deqn{\hat p_{Y,w}=\dfrac{\sum_{i\in S} w_i\cdot I(y_i=1)}{\sum_{i\in S} w_i}.}
#'
#' See Iparragirre et al. (2022) and Lopez-Raton et al. (2014) for more information.
#'
#'
#' @references Iparragirre, A., Barrio, I., Aramendi, J. and Arostegui, I. (2022). Estimation of cut-off points under complex-sampling design data. \emph{SORT-Statistics and Operations Research Transactions} \bold{46}(1), 137--158.
#' @references Lopez-Raton, M., Rodriguez-Alvarez, M.X, Cadarso-Suarez, C. and Gude-Sampedro, F. (2014). OptimalCutpoints: An R Package for Selecting Optimal Cutpoints in Diagnostic Tests. \emph{Journal of Statistical Software} \bold{61}(8), 1--36.
#'
#' @returns The output of this function is an object of class `wocp`. This object is a list that contains information about the following 4 elements:
#' - `tags`: a list containing two elements with the following information:
#'   - `tag.event`: a character string indicating the event of interest.
#'   - `tag.nonevent`: a character string indicating the non-event.
#' - `basics`: a list containing information of the following 4 elements:
#'   - `n.event`: number of units with the event of interest in the data set.
#'   - `n.nonevent`: number of units without the event of interest in the data set.
#'   - `hatN.event`: number of units with the event of interest represented in the population by all the event units in the data set, i.e., the sum of the sampling weights of the units with the event of interest in the data set.
#'   - `hatN.nonevent`: a numeric value indicating the number of non-event units in the population represented by means of the non-event units in the data set, i.e., the sum of the sampling weights of the non-event units in the data set.
#' - `optimal.cutoff`: this object is a list of three elements containing the information described below:
#'   - `method`: a character string indicating the method implemented to select the optimal cut-off point.
#'   - `optimal`: a list containing information of the following four elements:
#'      - `cutoff`: a numeric vector indicating the optimal cut-off point(s) that optimize(s) the selected criterion.
#'      - `Sew`: a numeric vector indicating the estimated sensitivity parameter(s) corresponding to the optimal cut-off point(s) that optimize(s) the selected criterion.
#'      - `Spw`: a numeric vector indicating the estimated specificity parameter(s) corresponding to the optimal cut-off point(s) that optimize(s) the selected criterion.
#'      - `criterion`: a numeric value indicating the criterion value optimized by means of the selected optimal cut-off point(s).
#'   - `all`: a list containing information on the following four elements:
#'      - `cutoff`: a numeric vector indicating all the cut-off points considered.
#'      - `Sew`: a numeric vector indicating the estimated sensitivity parameters corresponding to all the considered cut-off points.
#'      - `Spw`: a numeric vector indicating the estimated sensitivity parameters corresponding to all the considered cut-off points.
#'      - `criterion`: a numeric vector indicating the values of the selected criterion corresponding to all the considered cut-off points.
#' - `call`: an object saving the information about the way in which the function has been run.
#'
#' @export
#'
#' @examples
#' data(example_data_wroc)
#'
#' myocp <- wocp(response.var = "y", phat.var = "phat", weights.var = "weights",
#'               tag.event = 1, tag.nonevent = 0,
#'               method = "Youden",
#'               data = example_data_wroc)
#'
#'# Or equivalently
#' myocp <- wocp(example_data_wroc$y, example_data_wroc$phat, example_data_wroc$weights,
#'               tag.event = 1, tag.nonevent = 0, method = "Youden")
#'
#'
wocp <- function(response.var, phat.var, weights.var = NULL,
                 tag.event = NULL, tag.nonevent = NULL,
                 method = c("Youden", "MaxProdSpSe", "ROC01", "MaxEfficiency"),
                 data = NULL, design = NULL){

  if(missing(method) || is.null(method)) {
    stop("'method' argument required.")
  }
  if(any(!(method %in% c("MaxProdSpSe","ROC01","Youden","MaxEfficiency")))) {
    stop ("You have entered an invalid method.")
  }

  if (missing(phat.var)|| is.null(phat.var)) {
    stop("'phat.var' argument required.")
  }
  if (missing(response.var)|| is.null(response.var)) {
    stop("'response.var' argument required.")
  }

  if(inherits(response.var, "character")){
    response.var <- data[,response.var]
  }

  if(length(table(response.var))!=2){stop("Response variable must have two classes.")}

  if(inherits(phat.var, "character")){
    phat.var <- data[,phat.var]
  }

  if(inherits(weights.var, "character")){
    weights.var <- data[,weights.var]
  }

  if(!is.null(design)){
    data <- get(design$call$data)
    weights <- as.character(design$call$weights[2])
    weights.var <- data[,weights]
  }

  if(length(response.var) != length(phat.var) || length(response.var) != length(weights.var)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }

  # Missing tags:
  if(is.null(tag.nonevent)){
    if(is.null(tag.event)){
      tag.nonevent <- names(table(response.var)[which.max(table(response.var))])
      tag.event <- names(table(response.var)[which.min(table(response.var))])
    } else {
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.event)
      tag.nonevent <- tags[-tag.other]
    }
  } else {
    if(is.null(tag.event)){
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.nonevent)
      tag.event <- tags[-tag.other]
    }
  }

  res <- list()
  res$optimal.cutoff <- list()
  res$optimal.cutoff$method <- method

  pop.prev <- sum(weights.var[which(response.var != tag.nonevent)])/sum(weights.var)
  cutoffs <- sort(unique(phat.var))
  sew.v <- sapply(lapply(cutoffs, wse, response.var = response.var, tag.event = tag.event, phat.var = phat.var, weights.var = weights.var), "[[", 1)
  spw.v <- sapply(lapply(cutoffs, wsp, response.var = response.var, tag.nonevent = tag.nonevent, phat.var = phat.var, weights.var = weights.var), "[[", 1)

  # Optimal cut-off point calculation:
  criterion.v <- criterion.function(method = method, pop.prev = pop.prev, se.v = sew.v, sp.v = spw.v)
  opt.ocp <- optimize.criterion(method = method, criterion.v = criterion.v, se.v = sew.v, sp.v = spw.v, cutoffs = cutoffs)

  res <- list()

  res$tags <- list(tag.event = as.character(tag.event),
                   tag.nonevent = as.character(tag.nonevent))
  res$basics <- list(n.event = length(which(response.var==tag.event)),
                     n.nonevent = length(which(response.var==tag.nonevent)),
                     hatN.event = sum(weights.var[which(response.var==tag.event)]),
                     hatN.nonevent = sum(weights.var[which(response.var==tag.nonevent)]))

  res$optimal.cutoff <- list()
  res$optimal.cutoff$method <- method
  res$optimal.cutoff$optimal <- opt.ocp$optimal
  res$optimal.cutoff$all <- opt.ocp$all

  res$call <- match.call()

  class(res) <- "wocp"

  return(res)

}

#' Simulated data
#'
#' This dataset has been simulated in order to provide the users with an example dataset.
#'
#' @format ## `example_variables_wroc_2`
#' A data frame with 1720 rows and 10 columns:
#' \describe{
#'   \item{y}{Response variable}
#'   \item{x1,...,x6}{Covariates}
#'   \item{strata}{Strata variable}
#'   \item{cluster}{Cluster variable}
#'   \item{weights}{Sampling weights}
#'   ...
#' }
"example_variables_wroc_2"

#' Design-based confidence intervals for the AUC.
#' 
#' @param design An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data.
#' @param response.var A character string with the name of the column indicating the response variable in the data set.
#' @param phat.var A character string with the name of the column indicating the estimated probabilities in the data set.
#' @param tag.event A character string indicating the label used to indicate the event of interest in \code{response.var}.
#' The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.
#' @param tag.nonevent A character string indicating the label used for non-event in \code{response.var}.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param method A character string indicating the method implemented to calculate the confidence interval: 
#' \code{method = 'JKn'} or \code{method = 'subbootstrap'}. Default is \code{method = 'JKn'}.
#' @param boot.B A numeric value indicating the number of bootstrap resamples to be considered. Default is \code{boot.B=1000}. Only for \code{method = 'subbootstrap'}. 
#' @param boot.ci Select \code{boot.ci = 'normal'} for standardized normal distribution-based confidence intervals 
#' and \code{boot.ci = 'quantile'} for confidence intervals based on quantiles. Default is \code{boot.ci = 'normal'}. Only for \code{method = 'subbootstrap'}.     
#' @param conf.level A numeric value between 0 and 1 indicating the confidence level. Default is \code{conf.level = 0.95}, resulting in a 95% CI.
#'
#' @return The output object of this function is a list of 2 elements containing the following information:
#' - `wauc`: a numeric value indicating the weighted point estimate of the AUC.
#' - `ci`: a numeric vector indicating the lower and upper values of the confidence interval.
#' 
#' @importFrom stats qnorm
#' @importFrom stats quantile
#' @importFrom stats weights
#' 
#' @references Iparragirre, A., Lumley, T., & Barrio, I. (2026). 
#' Design-Based Inference for the AUC with Complex Survey Data. 
#' arXiv, 2603.28320. 
#' (https://arxiv.org/abs/2603.28320)
#' 
#' @export
#'
#' @examples
#' \donttest{
#' mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights, 
#' data = example_variables_wroc, nest=TRUE)
#' m <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = mydesign, family = quasibinomial())
#' phat <- predict(m, newdata = example_variables_wroc, type = "response")
#' example_variables_wroc$phat <- phat
#' mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights, 
#' data = example_variables_wroc, nest=TRUE)
#' 
#' # JKn-based confidence interval:
#' ci.JKn <- ci.wauc(design=mydesign, response.var = "y", phat.var = "phat", 
#'                   tag.event = 1, tag.nonevent = 0, method="JKn")
#' 
#' # Bootstrap-based confidence interval:
#' set.seed(123)
#' ci.boot <- ci.wauc(design=mydesign, response.var = "y", phat.var = "phat", 
#'                    tag.event = 1, tag.nonevent = 0, method="subbootstrap")
#' }
#' 

ci.wauc <- function(design, response.var, phat.var, tag.event = NULL, tag.nonevent = NULL,
                    method = c("JKn","subbootstrap"), boot.B=1000, boot.ci = c("normal", "quantile"), 
                    conf.level = 0.95){
  
  data <- get(design$call$data)
  weights <- as.character(design$call$weights[2])
  weights.var <- data[,weights]
  
  if(inherits(response.var, "character")){
    response.var <- data[,response.var]
  }
  
  if(length(table(response.var))!=2){stop("Response variable must have two classes.")}
  
  if(inherits(phat.var, "character")){
    phat.var <- data[,phat.var]
  }
  
  if(length(response.var) != length(phat.var) || length(response.var) != length(weights.var)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }
  
  # Missing tags:
  if(is.null(tag.nonevent)){
    if(is.null(tag.event)){
      tag.nonevent <- names(table(response.var)[which.max(table(response.var))])
      tag.event <- names(table(response.var)[which.min(table(response.var))])
    } else {
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.event)
      tag.nonevent <- tags[-tag.other]
    }
  } else {
    if(is.null(tag.event)){
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.nonevent)
      tag.event <- tags[-tag.other]
    }
  }
  
  method <- match.arg(method)
  boot.ci <- match.arg(boot.ci)
  
  alpha <- 1-conf.level
  
  # Point estimate:
  wauc <- wauc(response.var = response.var, phat.var = phat.var, weights.var = weights.var, tag.event = tag.event, tag.nonevent = tag.nonevent)$AUCw[1]
  
  if(method == "subbootstrap"){
    RB.boot.w <- weights(survey::as.svrepdesign(design = design, type = method, replicates=boot.B),"analysis")
    RB.aucw.b <- as.vector(apply(
      RB.boot.w, 2,
      function(x) wauc(
        response.var = response.var,
        phat.var = phat.var,
        weights.var = x,
        tag.event = tag.event,
        tag.nonevent = tag.nonevent
      )$AUCw[1]
    ))
    if(boot.ci == "normal"){
      var.RB <- var(RB.aucw.b)
      ci.l <- wauc - qnorm(alpha/2, lower.tail = FALSE)*sqrt(var.RB)
      ci.h <- wauc + qnorm(alpha/2, lower.tail = FALSE)*sqrt(var.RB)
    } else {
      if(boot.ci == "quantile"){
        ci.l <- as.vector(quantile(RB.aucw.b, probs = c(alpha/2)))
        ci.h <- as.vector(quantile(RB.aucw.b, probs = c(1-alpha/2)))
      }
    }
  } else {
    if(method == "JKn"){
      JKn.w <- weights(survey::as.svrepdesign(design = design, type = method),"analysis")
      JKn.aucw.1out <- as.vector(apply(
        JKn.w, 2,
        function(x) wauc(
          response.var = response.var,
          phat.var = phat.var,
          weights.var = x,
          tag.event = tag.event,
          tag.nonevent = tag.nonevent
        )$AUCw[1]
      ))
      strata_var <- attr(design$strata, "names")
      psu_var <- attr(design$cluster, "names")
      nclus.h <- table(unique(design$variables[,c(strata_var, psu_var)])[[strata_var]])
      id.0 <- apply(JKn.w==0, 2, which)
      strata_ids <- sapply(id.0, function(indexes) {
        unique(design$variables[indexes, strata_var])
      })
      ah <- nclus.h[as.character(strata_ids)]
      JKn.var <- sum(((ah-1)/ah)*((JKn.aucw.1out-wauc)^2))
      ci.l <- wauc - qnorm(alpha/2, lower.tail = FALSE)*sqrt(JKn.var)
      ci.h <- wauc + qnorm(alpha/2, lower.tail = FALSE)*sqrt(JKn.var)
    } else {
      stop("One of the following methods must be selected: method = 'JKn' or method = 'subbootstrap'.")
    }
  }
  
  obj <- list(wauc = wauc,
              ci = c(ci.l, ci.h))
  
  return(obj)
  
}

# AUC correction methods:

# -------------------------------------------------------------------------
# --> dCV
# -------------------------------------------------------------------------

cor.dCV <- function(data,
                    response.var, weights.var, strata.var, cluster.var, k = 10, R = 1, method = c("average", "pooling"),
                    formula, tag.event, tag.nonevent){

  if(method == "average"){
    dCV.sw.test = FALSE
  }
  if(method == "pooling"){
    dCV.sw.test = TRUE
  }

  rw.data <- svyVarSel::replicate.weights(data, method = "dCV",
                                       cluster = cluster.var, strata = strata.var, weights = weights.var,
                                       k = k, R = R, rw.test = TRUE, dCV.sw.test = dCV.sw.test)

  if(method == "average"){

    test.aucs <- matrix(NA, nrow = R, ncol = k)
    for(rr in 1:R){
      for(kk in 1:k){
        train.weights <- paste0("rw_r_",rr,"_train_",kk)
        test.weights <- paste0("rw_r_",rr,"_test_",kk)
        if(is.null(cluster.var)){
          sampling.design <- survey::svydesign(id=~1, strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
        } else {
          sampling.design <- survey::svydesign(id=~get(cluster.var), strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
        }
        suppressWarnings(train.model <- survey::svyglm(formula, design=sampling.design, family=quasibinomial()))
        phat.var <- as.numeric(predict(train.model, newdata = rw.data, type="response"))
        test.aucs[rr,kk] <- wauc(response.var = rw.data[,response.var], phat.var = phat.var, weights.var = rw.data[,test.weights], tag.event = tag.event, tag.nonevent = tag.nonevent)$AUCw
      }
    }

  }

  if(method == "pooling"){

    test.aucs <- rep(NA, R)

    for(rr in 1:R){

      p.hat <- rep(NA, nrow(data))

      for(kk in 1:k){
        train.weights <- paste0("rw_r_",rr,"_train_",kk)
        test.weights <- paste0("sw_r_",rr,"_test_",kk)
        if(is.null(cluster.var)){
          sampling.design <- survey::svydesign(id=~1, strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
        } else {
          sampling.design <- survey::svydesign(id=~get(cluster.var), strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
        }
        suppressWarnings(train.model <- survey::svyglm(formula, design=sampling.design, family=quasibinomial()))
        p.hat[which(rw.data[,test.weights]!=0)] <- train.model$fitted.values[which(rw.data[,test.weights]!=0)]
      }

      test.aucs[rr] <- wauc(response.var = data[,response.var],
                            phat.var = p.hat,
                            weights.var = data[,weights.var],
                            tag.event = tag.event, tag.nonevent = tag.nonevent)$AUCw

    }

  }

  corrected.AUC <- mean(test.aucs)
  return(corrected.AUC)

}



# -------------------------------------------------------------------------
# --> JKn
# -------------------------------------------------------------------------

cor.JKn <- function(data, response.var, weights.var, strata.var, cluster.var,
                    formula, tag.event, tag.nonevent){

  rw.data <- svyVarSel::replicate.weights(data, method = "JKn",
                                       cluster = cluster.var, strata = strata.var, weights = weights.var,
                                       rw.test = TRUE)

  p.hat <- rep(NA, nrow(data))
  rwtraincols <- grep("rw_r_1_train_", colnames(rw.data))
  k <- length(rwtraincols)

  for(kk in 1:k){
    train.weights <- paste0("rw_r_1_train_",kk)
    test.weights <- paste0("sw_r_1_test_",kk)
    if(is.null(cluster.var)){
      sampling.design <- survey::svydesign(id=~1, strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
    } else {
      sampling.design <- survey::svydesign(id=~get(cluster.var), strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
    }
    suppressWarnings(train.model <- survey::svyglm(formula, design=sampling.design, family=quasibinomial()))
    p.hat[which(rw.data[,test.weights]!=0)] <- train.model$fitted.values[which(rw.data[,test.weights]!=0)]
  }

  corrected.AUC <- wauc(response.var = response.var, phat.var = p.hat, weights.var = weights.var, tag.event = tag.event, tag.nonevent = tag.nonevent, data = data)$AUCw

  return(corrected.AUC)

}


# -------------------------------------------------------------------------
# --> RB
# -------------------------------------------------------------------------

cor.RB <- function(data,
                   response.var, weights.var, strata.var, cluster.var,
                   B = 200,
                   formula,
                   method = c("subbootstrap", "bootstrap"),
                   tag.event, tag.nonevent){

  if(is.null(cluster.var)){
    sampling.design <- survey::svydesign(id=~1, strata=~get(strata.var), weights=~get(weights.var), data=data, nest=TRUE)
  } else {
    sampling.design <- survey::svydesign(id=~get(cluster.var), strata=~get(strata.var), weights=~get(weights.var), data=data, nest=TRUE)
  }
  s.model <- survey::svyglm(formula, design=sampling.design, family=quasibinomial())
  auc.w.app <- wauc(response.var = s.model$y, phat.var = s.model$fitted.values, weights.var = data[,weights.var])$AUCw

  rw.data <- svyVarSel::replicate.weights(data, method = method,
                                       cluster = cluster.var, strata = strata.var, weights = weights.var,
                                       B = B, rw.test = TRUE)

  opt <- rep(NA, B)
  for(b in 1:B){
    # Bootstrap laginetako auc.w.app.b
    train.weights <- paste0("rw_r_1_train_",b)
    if(is.null(cluster.var)){
      b.design <- survey::svydesign(id=~1, strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
    } else {
      b.design <- survey::svydesign(id=~get(cluster.var), strata=~get(strata.var), weights=~get(train.weights), data=rw.data, nest=TRUE)
    }
    suppressWarnings(b.model <- survey::svyglm(formula, design=b.design, family=quasibinomial()))
    if(sum(is.na(b.model$coefficients)) > 0){
      b.model$coefficients[which(is.na(b.model$coefficients))] <- 0
      opt[b] <- NA
    } else {
      phat.var <- as.numeric(predict(b.model, newdata = rw.data, type="response"))
      auc.w.boot.b <- wauc(response.var = rw.data[,response.var], phat.var = phat.var, weights.var = rw.data[,train.weights])$AUCw
      auc.w.boot.s <- wauc(response.var = rw.data[,response.var], phat.var = phat.var, weights.var = rw.data[,weights.var])$AUCw
      opt[b] <- auc.w.boot.b - auc.w.boot.s
    }

  }

  o <- mean(opt)
  auc.RB <- auc.w.app - o

}

#' Estimation of the AUC of logistic regression models with complex survey data.
#'
#' Calculate the AUC of a logistic regression model
#' considering sampling weights with complex survey data
#' @param response.var A character string with the name of the column indicating the response variable in the data set
#' or a vector (either numeric or character string) with information of the response variable for all the units.
#' @param phat.var A character string with the name of the column indicating the estimated probabilities in the data set
#' or a numeric vector containing estimated probabilities for all the units.
#' @param weights.var A character string indicating the name of the column with sampling weights or
#' a numeric vector containing information of the sampling weights.
#' It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
#' For unweighted estimates, set all the sampling weight values to 1.
#' @param tag.event A character string indicating the label used to indicate the event of interest in \code{response.var}.
#' The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.
#' @param tag.nonevent A character string indicating the label used for non-event in \code{response.var}.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param data A data frame which, at least, must incorporate information on the columns
#' \code{response.var}, \code{phat.var} and \code{weights.var}.
#' If \code{data=NULL}, then specific numerical vectors must be included in
#' \code{response.var}, \code{phat.var} and \code{weights.var},
#' or the sampling design should be indicated in the argument \code{design}.
#' @param design An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data.
#' If \code{design = NULL}, information on the data set (argument \code{data})
#' and/or sampling weights (argument \code{weights.var}) must be included.
#'
#' @return The output object of this function is a list of 4 elements containing the following information:
#' - `AUCw`: the weighted estimate of the AUC.
#' - `tags`: a list containing two elements with the following information:
#'    - `tag.event`: a character string indicating the event of interest.
#'    - `tag.nonevent`: a character string indicating the non-event.
#' - `basics`: a list containing information of the following 4 elements:
#'    - `n.event`: number of units with the event of interest in the data set.
#'    - `n.nonevent`: number of units without the event of interest in the data set.
#'    - `hatN.event`: number of units with the event of interest represented in the population by all the event units in the data set, i.e., the sum of the sampling weights of the units with the event of interest in the data set.
#'    - `hatN.nonevent`: a numeric value indicating the number of non-event units in the population represented by means of the non-event units in the data set, i.e., the sum of the sampling weights of the non-event units in the data set.
#' - `call`: an object saving the information about the way in which the function has been run.
#'
#' @details
#' \eqn{S} indicate a sample of \eqn{n} observations of the vector of random variables \eqn{(Y,\pmb X)}, and \eqn{\forall i=1,\ldots,n,} \eqn{y_i} indicate the \eqn{i^{th}} observation of the response variable \eqn{Y},
#' and \eqn{\pmb x_i} the observations of the vector covariates \eqn{\pmb X}. Let \eqn{w_i} indicate the sampling weight corresponding to the unit \eqn{i} and \eqn{\hat p_i} the estimated probability of event.
#' Let \eqn{S_0} and \eqn{S_1} be subsamples of \eqn{S}, formed by the units without the event of interest (\eqn{y_i=0}) and with the event of interest (\eqn{y_i=1}), respectively.
#' Then, the AUC is estimated as follows:
#' \deqn{\widehat{AUC}_w=\dfrac{\sum_{j\in S_0}\sum_{k\in S_1}w_jw_k \{I(\hat p_j < \hat p_k) + 0.5\cdot I(\hat p_j = \hat p_k)\}}{\sum_{j\in S_0}\sum_{k\in S_1}w_jw_k}.}
#' See Iparragirre et al (2023) for more information.
#'
#' @references Iparragirre, A., Barrio, I. and Arostegui, I. (2023).
#' Estimation of the ROC curve and the area under it with complex survey data.
#' *Stat* **12**(1), e635. (https://doi.org/10.1002/sta4.635)
#'
#' @export
#'
#' @examples
#' data(example_data_wroc)
#'
#' auc.obj <- wauc(response.var = "y",
#'                 phat.var = "phat",
#'                 weights.var = "weights",
#'                 tag.event = 1,
#'                 tag.nonevent = 0,
#'                 data = example_data_wroc)
#'
#' # Or equivalently
#' auc.obj <- wauc(response.var = example_data_wroc$y,
#'                 phat.var = example_data_wroc$phat,
#'                 weights.var = example_data_wroc$weights,
#'                 tag.event = 1, tag.nonevent = 0)
#'
wauc <- function(response.var, phat.var, weights.var = NULL, tag.event = NULL, tag.nonevent = NULL, data = NULL, design = NULL){

  if(!is.null(design)){
    data <- get(design$call$data)
    weights <- as.character(design$call$weights[2])
    weights.var <- data[,weights]
  }
  
  if(inherits(response.var, "character")){
    response.var <- data[,response.var]
  }

  if(length(table(response.var))!=2){stop("Response variable must have two classes.")}
  

  if(inherits(phat.var, "character")){
    phat.var <- data[,phat.var]
  }

  if(inherits(weights.var, "character")){
    weights.var <- data[,weights.var]
  }

  if(length(response.var) != length(phat.var) || length(response.var) != length(weights.var)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }

  # Missing tags:
  if(is.null(tag.nonevent)){
    if(is.null(tag.event)){
      tag.nonevent <- names(table(response.var)[which.max(table(response.var))])
      tag.event <- names(table(response.var)[which.min(table(response.var))])
    } else {
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.event)
      tag.nonevent <- tags[-tag.other]
    }
  } else {
    if(is.null(tag.event)){
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.nonevent)
      tag.event <- tags[-tag.other]
    }
  }


  p0 <- phat.var[which(response.var==tag.nonevent)]
  p1 <- phat.var[which(response.var==tag.event)]

  w0 <- weights.var[which(response.var==tag.nonevent)]
  w1 <- weights.var[which(response.var==tag.event)]

  auc <- (sum(outer(w1, w0, "*")[which(outer(p1, p0, ">"))]) + 0.5*sum(outer(w1, w0, "*")[which(outer(p1, p0, "=="))]))/(sum(w0)*sum(w1))


  auc.obj <- list()
  auc.obj$AUCw <- auc
  auc.obj$tags <- list(tag.event = as.character(tag.event),
                       tag.nonevent = as.character(tag.nonevent))
  auc.obj$basics <- list(n.event = length(which(response.var==tag.event)),
                         n.nonevent = length(which(response.var==tag.nonevent)),
                         hatN.event = sum(w1),
                         hatN.nonevent = sum(w0))
  auc.obj$call <- match.call()


  return(auc.obj)

}

#' Corrected estimate of the AUC based on replicate weights.
#'
#' Optimism correction of the AUC of logistic regression models
#' with complex survey data based on replicate weights methods.
#' @param data A data frame which, at least, must incorporate information on the columns
#' \code{response.var}, \code{phat.var} and \code{weights.var}.
#' If \code{data=NULL}, the sampling design must be indicated in the argument \code{design}.
#' @param formula Formula of the model for which the AUC needs to be corrected.
#' The models are fitted by means of \code{survey::svyglm()} function.
#' @param tag.event A character string indicating the label used to indicate the event of interest in \code{response.var}.
#' The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.
#' @param tag.nonevent A character string indicating the label used for non-event in \code{response.var}.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param weights.var A character string indicating the name of the column with sampling weights.
#' It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
#' @param strata.var A character string indicating the name of the column with strata identifiers.
#' It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
#' @param cluster.var A character string indicating the name of the column with cluster identifiers.
#' It could be \code{NULL} if the sampling design is indicated in the \code{design} argument
#' or the sampling design does not have considered clustering.
#' @param design An object of class \code{survey.design} generated by \code{survey::svydesign()}.
#' It could be \code{NULL} if information about \code{cluster.var}, \code{strata.var}, \code{weights.var} and \code{data} are given.
#' @param method A character string indicating the method to be applied to define replicate weights and correct the AUC.
#' Choose between: \code{JKn} (for the Jackknife Repeated Replication), \code{dCV} (for the design-based cross-validation),
#' \code{RB} (for the Rescaling Bootstrap).
#' @param dCV.method Only applies for the \code{dCV} method.
#' Choose between: \code{average} (for the averaging cross-validation) or \code{pooling} (for the pooling cross-validation).
#' Note: \code{pooling} is recommended over \code{average} (see, Iparragirre and Barrio (2024))
#' @param RB.method Only applies for the \code{RB} method.
#' Choose between: \code{subbootstrap} or \code{bootstrap} (see the documentation of \code{svyVarSel::replicate.weights()} for help).
#' @param k A numeric value indicating the number of folds to be defined.
#' Default is \code{k=10}. Only applies for the \code{dCV} method.
#' @param R A numeric value indicating the number of times the sample is partitioned. Default is \code{R=1}. Only applies for \code{dCV}, \code{split} or \code{extrapolation} methods.
#' @param B A numeric value indicating the number of bootstrap resamples. Default is \code{B=200}. Only applies for \code{bootstrap} and  \code{subbootstrap} methods.
#'
#' @details
#' See Iparragirre and Barrio (2024) for more information on the AUC correction methods and their performance.
#'
#'
#' @references Iparragirre, A., Barrio, I. (2024).
#' Optimism Correction of the AUC with Complex Survey Data.
#' In: Einbeck, J., Maeng, H., Ogundimu, E., Perrakis, K. (eds)
#' Developments in Statistical Modelling. IWSM 2024. Contributions to Statistics.
#' Springer, Cham. https://doi.org/10.1007/978-3-031-65723-8_7
#'
#' @return The output object of this function is a list of 5 elements containing the following information:
#' - `corrected.AUCw`: the corrected estimate of the weighted AUC.
#' - `correction.method`: the selected correction method.
#' - `formula`: formula of the model that has been fitted.
#' - `tags`: a list containing two elements with the following information:
#'    - `tag.event`: a character string indicating the event of interest.
#'    - `tag.nonevent`: a character string indicating the non-event.
#' - `call`: an object saving the information about the way in which the function has been run.
#'
#' @import survey
#' @import svyVarSel
#' @importFrom stats predict
#' @importFrom stats quasibinomial
#'
#' @export
#'
#' @examples
#' data(example_variables_wroc)
#' mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata,
#'                               weights = ~weights, nest = TRUE,
#'                               data = example_variables_wroc)
#' m <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = mydesign,
#'                     family = quasibinomial())
#' phat <- predict(m, newdata = example_variables_wroc, type = "response")
#' myaucw <- wauc(response.var = example_variables_wroc$y, phat.var = phat,
#'                weights.var = example_variables_wroc$weights)
#'
#' # Correction of the AUCw:
#' set.seed(1)
#' res <- corrected.wauc(data = example_variables_wroc,
#'                       formula = y ~ x1 + x2 + x3 + x4 + x5 + x6,
#'                       tag.event = 1, tag.nonevent = 0,
#'                       weights.var = "weights", strata.var = "strata", cluster.var = "cluster",
#'                       method = "dCV", dCV.method = "pooling", k = 10, R = 20)
#' # Or equivalently:
#' \donttest{
#' set.seed(1)
#' res <- corrected.wauc(design = mydesign,
#'                       formula = y ~ x1 + x2 + x3 + x4 + x5 + x6,
#'                       tag.event = 1, tag.nonevent = 0,
#'                       method = "dCV", dCV.method = "pooling", k = 10, R = 20)
#' }
#'
corrected.wauc <- function(data = NULL, formula, tag.event = NULL, tag.nonevent = NULL,
                           weights.var = NULL, strata.var = NULL, cluster.var = NULL, design = NULL,
                           method = c("dCV", "JKn", "RB"),
                           dCV.method = c("average", "pooling"), RB.method = c("subbootstrap", "bootstrap"),
                           k = 10, R = 1, B = 200){

  # Identify the response variable:
  response.var <- as.character(formula[2])

  if(!is.null(design)){
    data <- get(design$call$data)
    weights.var <- as.character(design$call$weights[2])
    strata.var <- as.character(design$call$strata[2])
    cluster.var <- as.character(design$call$ids[2])
  }

  if(length(table(data[,response.var]))!=2){stop("Response variable must have two classes.")}


  # Missing tags:
  if(is.null(tag.nonevent)){
    if(is.null(tag.event)){
      tag.nonevent <- names(table(data[,response.var])[which.max(table(data[,response.var]))])
      tag.event <- names(table(data[,response.var])[which.min(table(data[,response.var]))])
    } else {
      tags <- names(table(data[,response.var]))
      tag.other <- which(tags == tag.event)
      tag.nonevent <- tags[-tag.other]
    }
  } else {
    if(is.null(tag.event)){
      tags <- names(table(data[,response.var]))
      tag.other <- which(tags == tag.nonevent)
      tag.event <- tags[-tag.other]
    }
  }


  # Methods:
  if(length(method)!=1){
    stop("Please, select one (and only one) correction method: 'dCV', 'JKn' or 'RB'. See the documentation of the package for help.")
  }

  if(method == "dCV"){
    if(length(dCV.method) != 1){
      stop("Please, select one (and only one) correction method: 'average' or 'pooling'. See the documentation of the package for help.")
    } else {
      if(dCV.method != "average" & dCV.method != "pooling"){
        stop("Please, select one (and only one) correction method: 'average' or 'pooling'. See the documentation of the package for help.")
      } else {
        auc <- cor.dCV(data = data,
                       response.var = response.var,
                       weights.var = weights.var,
                       strata.var = strata.var,
                       cluster.var = cluster.var,
                       k = k, R = R,
                       method = dCV.method,
                       formula = formula,
                       tag.event = tag.event,
                       tag.nonevent = tag.nonevent)
      }
    }
  }

  if(method == "JKn"){
    auc <- cor.JKn(data = data,
                   response.var = response.var,
                   weights.var = weights.var,
                   strata.var = strata.var,
                   cluster.var = cluster.var,
                   formula = formula,
                   tag.event = tag.event,
                   tag.nonevent = tag.nonevent)
  }

  if(method == "RB"){
    if(length(RB.method) != 1){
      stop("Please, select one (and only one) correction method: 'subbootstrap' or 'bootstrap'. See the documentation of the package for help.")
    } else {
      if(RB.method != "subbootstrap" & RB.method != "bootstrap"){
        stop("Please, select one (and only one) correction method: 'subbootstrap' or 'bootstrap'. See the documentation of the package for help.")
      } else {
        auc <- cor.RB(data = data,
                      response.var = response.var,
                      weights.var = weights.var,
                      strata.var = strata.var,
                      cluster.var = cluster.var,
                      B = B,
                      formula = formula,
                      method = RB.method,
                      tag.event = tag.event,
                      tag.nonevent = tag.nonevent)
     }
    }
  }

  auc.obj <- list()
  auc.obj$corrected.AUCw <- auc
  auc.obj$correction.method <- list()
  auc.obj$correction.method$method <- method
  if(method == "dCV"){
    auc.obj$correction.method$dCV.method <- dCV.method
    auc.obj$correction.method$k <- k
    auc.obj$correction.method$R <- R
  }
  if(method == "RB"){
    auc.obj$correction.method$RB.method <- RB.method
    auc.obj$correction.method$B <- B
  }
  auc.obj$formula <- formula
  auc.obj$tags <- list(tag.event = as.character(tag.event),
                       tag.nonevent = as.character(tag.nonevent))
  auc.obj$call <- match.call()


  return(auc.obj)


}

#' Design-based hypothesis test for the comparison of two independent AUCs.
#'
#' @param design.1 An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data set 1.
#' @param response.var.1 A character string with the name of the column indicating the response variable in the data set 1.
#' @param phat.var.1 A character string with the name of the column indicating the estimated probabilities in the data set 1.
#' @param design.2 An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data set 2.
#' @param response.var.2 A character string with the name of the column indicating the response variable in the data set 2.
#' @param phat.var.2 A character string with the name of the column indicating the estimated probabilities in the data set 2.
#' @param tag.event A character string indicating the label used to indicate the event of interest.
#' The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.
#' @param tag.nonevent A character string indicating the label used for non-event.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param alternative A character string specifying the alternative hypothesis: \code{'two.sided'}, \code{greater}, or \code{less}. 
#' Default is \code{alternative = 'two.sided'}.
#' @param method A character string indicating the method implemented to estimate the variance: 
#' \code{method = 'JKn'} or \code{method = 'subbootstrap'}. Default is \code{method = 'JKn'}.
#' @param boot.B A numeric value indicating the number of bootstrap resamples to be considered. Default is \code{boot.B=1000}. Only for \code{method = 'subbootstrap'}. 
#'
#' @returns The output object of this function is a list of 5 elements containing the following information:
#' - `wauc.1`: a numeric value indicating the weighted point estimate of the AUC for model 1.
#' - `wauc.2`: a numeric value indicating the weighted point estimate of the AUC for model 2.
#' - `sd`: a numeric value indicating the square root of the estimated variance.
#' - `z`: a numeric value indicating the value of the statistic.
#' - `pvalue`: a numeric value indicating the p-value of the test.
#' 
#' @importFrom stats pnorm
#' @importFrom stats var
#' @importFrom stats weights
#' 
#' @references Iparragirre, A., Lumley, T., & Barrio, I. (2026). 
#' Design-Based Inference for the AUC with Complex Survey Data. 
#' arXiv, 2603.28320. 
#' (https://arxiv.org/abs/2603.28320)
#' 
#' @export
#'
#' @examples
#' \donttest{
#' des.1 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights, 
#'                            data = example_variables_wroc, nest=TRUE)
#' m.1 <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = des.1, family = quasibinomial())
#' example_variables_wroc$phat.1 <- predict(m.1, newdata = example_variables_wroc, type = "response")
#' des.1 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights,
#'                            data = example_variables_wroc, nest=TRUE)
#' des.2 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights, 
#'                            data = example_variables_wroc_2, nest=TRUE)
#' m.2 <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = des.2, family = quasibinomial())
#' example_variables_wroc$phat.2 <- predict(m.1, newdata = example_variables_wroc_2, type = "response")
#' des.2 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights,
#'                            data = example_variables_wroc_2, nest=TRUE)
#'                            
#' # JKn-based hypothesis test:
#' jkn.ht <- ht.indep(design.1=des.1, response.var.1="y", phat.var.1 = "phat.1",
#'                    design.2=des.2, response.var.2="y", phat.var.2 = "phat.2",
#'                    tag.event=1, tag.nonevent=0, method="JKn")
#'                    
#' # Bootstrap-based hypothesis test:
#' boot.ht <- ht.indep(design.1=des.1, response.var.1="y", phat.var.1 = "phat.1",
#'                     design.2=des.2, response.var.2="y", phat.var.2 = "phat.2",
#'                     tag.event=1, tag.nonevent=0, method="subbootstrap")
#' }
#'                     
ht.indep <- function(design.1, response.var.1, phat.var.1, 
                     design.2, response.var.2, phat.var.2, 
                     tag.event = NULL, tag.nonevent = NULL,
                     alternative = c("two.sided", "greater", "less"),
                     method = c("JKn","subbootstrap"), boot.B=1000){
  
  data.1 <- get(design.1$call$data)
  weights.1 <- as.character(design.1$call$weights[2])
  weights.var.1 <- data.1[,weights.1]
  
  data.2 <- get(design.2$call$data)
  weights.2 <- as.character(design.2$call$weights[2])
  weights.var.2 <- data.2[,weights.2]
  
  if(inherits(response.var.1, "character")){
    response.var.1 <- data.1[,response.var.1]
  }
  
  if(length(table(response.var.1))!=2){stop("Response variable must have two classes.")}
  
  if(inherits(phat.var.1, "character")){
    phat.var.1 <- data.1[,phat.var.1]
  }
  
  if(inherits(response.var.2, "character")){
    response.var.2 <- data.2[,response.var.2]
  }
  
  if(length(table(response.var.2))!=2){stop("Response variable must have two classes.")}
  
  if(inherits(phat.var.2, "character")){
    phat.var.2 <- data.2[,phat.var.2]
  }
  
  if(length(response.var.1) != length(phat.var.1) || length(response.var.1) != length(weights.var.1)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }
  
  if(length(response.var.2) != length(phat.var.2) || length(response.var.2) != length(weights.var.2)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }
  
  # Missing tags:
  if(is.null(tag.nonevent)){
    if(is.null(tag.event)){
      tag.nonevent <- names(table(response.var.1)[which.max(table(response.var.1))])
      tag.event <- names(table(response.var.1)[which.min(table(response.var.1))])
    } else {
      tags <- names(table(response.var.1))
      tag.other <- which(tags == tag.event)
      tag.nonevent <- tags[-tag.other]
    }
  } else {
    if(is.null(tag.event)){
      tags <- names(table(response.var.1))
      tag.other <- which(tags == tag.nonevent)
      tag.event <- tags[-tag.other]
    }
  }
  
  method <- match.arg(method)
  alternative <- match.arg(alternative)
  
  # Point estimates:
  wauc.1 <- wauc(response.var = response.var.1, phat.var = phat.var.1, weights.var = weights.var.1, 
                 tag.event = tag.event, tag.nonevent = tag.nonevent)$AUCw[1]
  wauc.2 <- wauc(response.var = response.var.2, phat.var = phat.var.2, weights.var = weights.var.2, 
                 tag.event = tag.event, tag.nonevent = tag.nonevent)$AUCw[1]
  
  if(method == "JKn"){
    JKn.w.1 <- stats::weights(survey::as.svrepdesign(design = design.1, type = method),"analysis")
    JKn.aucw.1 <- as.vector(apply(
      JKn.w.1, 2,
      function(x) wauc(
        response.var = response.var.1,
        phat.var = phat.var.1,
        weights.var = x,
        tag.event = tag.event,
        tag.nonevent = tag.nonevent
      )$AUCw[1]
    ))
    strata_var.1 <- attr(design.1$strata, "names")
    psu_var.1 <- attr(design.1$cluster, "names")
    nclus.h.1 <- table(unique(design.1$variables[,c(strata_var.1, psu_var.1)])[[strata_var.1]])
    id.0.1 <- apply(JKn.w.1==0, 2, which)
    strata_ids.1 <- sapply(id.0.1, function(indexes) {
      unique(design.1$variables[indexes, strata_var.1])
    })
    ah.1 <- nclus.h.1[as.character(strata_ids.1)]
    JKn.var.1 <- sum(((ah.1-1)/ah.1)*((JKn.aucw.1-wauc.1)^2))
    
    JKn.w.2 <- stats::weights(survey::as.svrepdesign(design = design.2, type = method),"analysis")
    JKn.aucw.2 <- as.vector(apply(
      JKn.w.2, 2,
      function(x) wauc(
        response.var = response.var.2,
        phat.var = phat.var.2,
        weights.var = x,
        tag.event = tag.event,
        tag.nonevent = tag.nonevent
      )$AUCw[1]
    ))
    strata_var.2 <- attr(design.2$strata, "names")
    psu_var.2 <- attr(design.2$cluster, "names")
    nclus.h.2 <- table(unique(design.2$variables[,c(strata_var.2, psu_var.2)])[[strata_var.2]])
    id.0.2 <- apply(JKn.w.2==0, 2, which)
    strata_ids.2 <- sapply(id.0.2, function(indexes) {
      unique(design.2$variables[indexes, strata_var.2])
    })
    ah.2 <- nclus.h.2[as.character(strata_ids.2)]
    JKn.var.2 <- sum(((ah.2-1)/ah.2)*((JKn.aucw.2-wauc.2)^2))
    
    d.sd <- sqrt(JKn.var.1+JKn.var.2)
    z <- (wauc.1 - wauc.2)/d.sd
    
  } else {
    
    if(method == "subbootstrap"){
      RB.boot.w.1 <- stats::weights(survey::as.svrepdesign(design = design.1, type = method, replicates=boot.B),"analysis")
      RB.aucw.b.1 <- as.vector(apply(
        RB.boot.w.1, 2,
        function(x) wauc(
          response.var = response.var.1,
          phat.var = phat.var.1,
          weights.var = x,
          tag.event = tag.event,
          tag.nonevent = tag.nonevent
        )$AUCw[1]
      ))
      RB.boot.w.2 <- stats::weights(survey::as.svrepdesign(design = design.2, type = method, replicates=boot.B),"analysis")
      RB.aucw.b.2 <- as.vector(apply(
        RB.boot.w.2, 2,
        function(x) wauc(
          response.var = response.var.2,
          phat.var = phat.var.2,
          weights.var = x,
          tag.event = tag.event,
          tag.nonevent = tag.nonevent
        )$AUCw[1]
      ))
      
      v.var1.RB <- var(RB.aucw.b.1)
      v.var2.RB <- var(RB.aucw.b.2)
      
      d.sd <- sqrt(v.var1.RB+v.var2.RB)
      z <- (wauc.1-wauc.2)/d.sd
      
    } else {
      stop("One of the following methods must be selected: method = 'JKn' or method = 'subbootstrap'.")
    }
    
  }
  
  if (alternative == "two.sided") {
    pval <- 2*pnorm(abs(z), lower.tail = FALSE)
  } else {
    if (alternative == "greater") {
      pval <- pnorm(z, lower.tail = FALSE)
    } else {
      if (alternative == "less") {
        pval <- pnorm(z, lower.tail = TRUE)
      } else {
        stop("One of the following options must be selected for the alternative hypothesis: 'two.sided', 'greater' or 'less'")
      }
    } 
  }
  
  obj <- list(wauc.1 = wauc.1,
              wauc.2 = wauc.2,
              sd = d.sd,
              z = z,
              pvalue = pval)
  
}

#' Estimation of the ROC curve of logistic regression models with complex survey data
#'
#' Plot the ROC curve of a logistic regression model
#' considering sampling weights with complex survey data.
#' @param x An object of class \code{wroc} obtained by means of the function \code{wroc()}.
#' @param print.auc A logical value. If \code{TRUE}, the value of the area under the ROCw curve (AUCw) is printed (default \code{print.auc = TRUE}).
#' @param print.cutoff A logical value. If \code{TRUE}, the value of the optimal cut-off point, and the corresponding weighted estimates of the sensitivity and specificity parameters are printed (default \code{print.cutoff = TRUE}).
#' @param col.cutoff A character string indicating the color in which the cut-off point is depicted. The default option is \code{col.cutoff = "red"}.
#' @param cex.text A numeric value indicating the size with which the information of the AUCw and optimal cut-off point is printed. The default option is \code{cex.text = 0.75}.
#' @param round.digits A numeric value indicating the number of digits that will be employed when printing the information about the AUCw and optimal cut-off point. The default option is \code{round.digits = 4}.
#'
#' @details
#' More information is given in the documentation of the \code{wroc()}, \code{wauc{}} and \code{wocp()} functions.
#'
#'
#' @return a graph
#'
#' @export
#'
#' @examples
#' data(example_data_wroc)
#'
#' mycurve <- wroc(response.var = "y", phat.var = "phat", weights.var = "weights",
#'                 data = example_data_wroc,
#'                 tag.event = 1, tag.nonevent = 0,
#'                 cutoff.method = "Youden")
#' wroc.plot(x = mycurve, print.auc = TRUE, print.cutoff = TRUE)
#'
wroc.plot <- function(x, print.auc = TRUE, print.cutoff = FALSE,
                      col.cutoff = "red", cex.text = 0.75, round.digits = 4){

  if(!inherits(x, "wroc")){stop("Please, insert an object of class 'wroc', obtained using the function wroc().")}

  if(print.cutoff == TRUE & is.null(x[["optimal.cutoff"]])){
    cat("The optimal cut-off point cannot be printed given that this information has not saved in the object 'x'.")
  }

  # Objects:
  spw.v <- x[["wroc.curve"]][["Spw.values"]]
  sew.v <- x[["wroc.curve"]][["Sew.values"]]

  inv.spw.v <- 1 - spw.v
  plot.df <- data.frame(sew.v, inv.spw.v)

  # Plot the graph:
  plot(x = plot.df$inv.spw.v, y = plot.df$sew.v, type = "l",
       xlab = "1-Spw(c)", ylab = "Sew(c)", main = "ROCw Curve")

  # Print AUCw:
  if(print.auc){
    graphics::text(x = 0.5, y = 0, cex = cex.text,
                   labels = paste0("Area Under the ROCw Curve (AUCw): " ,
                                   round(x[["wauc"]], digits = round.digits)))
  }

  # Plot the cut-off point:
  if(print.cutoff == TRUE & !is.null(x[["optimal.cutoff"]])){

    graphics::points(x = 1-x[["optimal.cutoff"]][["Spw"]],
                     y = x[["optimal.cutoff"]][["Sew"]],
                     col = col.cutoff, pch=19)
    graphics::text(x = 1-x[["optimal.cutoff"]][["Spw"]],
                   y = x[["optimal.cutoff"]][["Sew"]],
                   pos = 4, cex = cex.text,
                   labels = paste0(round(x[["optimal.cutoff"]][["cutoff.value"]], digits = round.digits),
                                   " (Spw: ", round(x[["optimal.cutoff"]][["Spw"]], digits = round.digits),
                                   ", Sew: ", round(x[["optimal.cutoff"]][["Sew"]], digits = round.digits), ")"))

  }

  graphics::abline(a = 0, b = 1, lty = 2, col = "gray")



}

criterion.function <- function(method = c("Youden", "MaxProdSpSe", "ROC01", "MaxEfficiency"), pop.prev = NULL, se.v, sp.v){

  criterion <- eval(parse(text = paste(method, ".criterion",sep = "")))(pop.prev = pop.prev, se.v = se.v, sp.v = sp.v)
  return(criterion)

}

Youden.criterion <- function(pop.prev = NULL, se.v, sp.v){

  criterion <- se.v + sp.v - 1
  return(criterion)

}

MaxProdSpSe.criterion <- function(pop.prev = NULL, se.v, sp.v){

  criterion <- sp.v * se.v
  return(criterion)

}


ROC01.criterion <- function(pop.prev = NULL, se.v, sp.v){

  criterion <- (sp.v-1)^2+(se.v-1)^2
  return(criterion)

}


MaxEfficiency.criterion <- function(pop.prev, se.v, sp.v){

  criterion <- pop.prev*se.v+(1-pop.prev)*sp.v
  return(criterion)

}

#' Design-based hypothesis test for the comparison of two paired AUCs.
#'
#' @param design An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data.
#' @param response.var A character string with the name of the column indicating the response variable in the data set.
#' @param phat.var.1 A character string with the name of the column indicating the estimated probabilities obtained with model 1 in the data set.
#' @param phat.var.2 A character string with the name of the column indicating the estimated probabilities obtained with model 2 in the data set.
#' @param tag.event A character string indicating the label used to indicate the event of interest in \code{response.var}.
#' The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.
#' @param tag.nonevent A character string indicating the label used for non-event in \code{response.var}.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param alternative A character string specifying the alternative hypothesis: \code{'two.sided'}, \code{greater}, or \code{less}. 
#' Default is \code{alternative = 'two.sided'}.
#' @param method A character string indicating the method implemented to estimate the variance: 
#' \code{method = 'JKn'} or \code{method = 'subbootstrap'}. Default is \code{method = 'JKn'}.
#' @param boot.B A numeric value indicating the number of bootstrap resamples to be considered. Default is \code{boot.B=1000}. Only for \code{method = 'subbootstrap'}. 
#'
#' @returns The output object of this function is a list of 5 elements containing the following information:
#' - `wauc.1`: a numeric value indicating the weighted point estimate of the AUC for model 1.
#' - `wauc.2`: a numeric value indicating the weighted point estimate of the AUC for model 2.
#' - `sd`: a numeric value indicating the square root of the estimated variance.
#' - `z`: a numeric value indicating the value of the statistic.
#' - `pvalue`: a numeric value indicating the p-value of the test.
#' 
#' @importFrom stats pnorm
#' @importFrom stats sd
#' @importFrom stats weights
#' 
#' @references Iparragirre, A., Lumley, T., & Barrio, I. (2026). 
#' Design-Based Inference for the AUC with Complex Survey Data. 
#' arXiv, 2603.28320. 
#' (https://arxiv.org/abs/2603.28320)
#' 
#' @export
#'
#' @examples
#' \donttest{
#' mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights,
#' data = example_variables_wroc, nest=TRUE)
#' model.1 <- survey::svyglm(y ~ x1 + x2 + x3, design = mydesign, family = quasibinomial())
#' model.2 <- survey::svyglm(y ~ x4 + x5 + x6, design = mydesign, family = quasibinomial())
#' phat.1 <- predict(model.1, newdata = example_variables_wroc, type = "response")
#' phat.2 <- predict(model.2, newdata = example_variables_wroc, type = "response")
#' example_variables_wroc$phat.1 <- phat.1
#' example_variables_wroc$phat.2 <- phat.2
#' mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights,
#' data = example_variables_wroc, nest=TRUE)
#' 
#' # JKn-based hypothesis test:
#' ht.JKn <- ht.paired(design=mydesign, response.var = "y", phat.var.1 = "phat.1",
#'                     phat.var.2 = "phat.2", tag.event = 1, tag.nonevent = 0, 
#'                     method="JKn")
#'                     
#' # Bootstrap-based hypothesis test:
#' set.seed(123)
#' ht.boot <- ht.paired(design=mydesign, response.var = "y", phat.var.1 = "phat.1",
#'                      phat.var.2 = "phat.2", tag.event = 1, tag.nonevent = 0, 
#'                      method="subbootstrap")
#' }
#' 
ht.paired <- function(design, response.var, phat.var.1, phat.var.2, 
                      tag.event = NULL, tag.nonevent = NULL,
                      alternative = c("two.sided", "greater", "less"),
                      method = c("JKn","subbootstrap"), boot.B=1000){
  
  data <- get(design$call$data)
  weights <- as.character(design$call$weights[2])
  weights.var <- data[,weights]
  
  if(inherits(response.var, "character")){
    response.var <- data[,response.var]
  }
  
  if(length(table(response.var))!=2){stop("Response variable must have two classes.")}
  
  if(inherits(phat.var.1, "character")){
    phat.var.1 <- data[,phat.var.1]
  }
  
  if(inherits(phat.var.2, "character")){
    phat.var.2 <- data[,phat.var.2]
  }
  
  if(length(response.var) != length(phat.var.1) || length(response.var) != length(phat.var.2) || length(response.var) != length(weights.var)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }
  
  # Missing tags:
  if(is.null(tag.nonevent)){
    if(is.null(tag.event)){
      tag.nonevent <- names(table(response.var)[which.max(table(response.var))])
      tag.event <- names(table(response.var)[which.min(table(response.var))])
    } else {
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.event)
      tag.nonevent <- tags[-tag.other]
    }
  } else {
    if(is.null(tag.event)){
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.nonevent)
      tag.event <- tags[-tag.other]
    }
  }
  
  method <- match.arg(method)
  alternative <- match.arg(alternative)
  
  # Point estimates:
  wauc.1 <- wauc(response.var = response.var, phat.var = phat.var.1, weights.var = weights.var, 
                 tag.event = tag.event, tag.nonevent = tag.nonevent)$AUCw[1]
  wauc.2 <- wauc(response.var = response.var, phat.var = phat.var.2, weights.var = weights.var, 
                 tag.event = tag.event, tag.nonevent = tag.nonevent)$AUCw[1]
 
  if(method == "JKn"){
    JKn.w <- stats::weights(survey::as.svrepdesign(design = design, type = method),"analysis")
    JKn.aucw.1 <- as.vector(apply(
      JKn.w, 2,
      function(x) wauc(
        response.var = response.var,
        phat.var = phat.var.1,
        weights.var = x,
        tag.event = tag.event,
        tag.nonevent = tag.nonevent
      )$AUCw[1]
    ))
    JKn.aucw.2 <- as.vector(apply(
      JKn.w, 2,
      function(x) wauc(
        response.var = response.var,
        phat.var = phat.var.2,
        weights.var = x,
        tag.event = tag.event,
        tag.nonevent = tag.nonevent
      )$AUCw[1]
    ))
    
    d.aucs <- wauc.1-wauc.2
    jkn.d.aucs <- JKn.aucw.1-JKn.aucw.2
    
    strata_var <- attr(design$strata, "names")
    psu_var    <- attr(design$cluster, "names")
    
    nclus.h <- table(unique(design$variables[,c(strata_var, psu_var)])[[strata_var]])
    id.0 <- apply(JKn.w==0, 2, which)
    strata_ids <- sapply(id.0, function(indexes) {
      unique(design$variables[indexes, strata_var])
    })
    ah <- nclus.h[as.character(strata_ids)]
    
    JKn.var <- sum(((ah-1)/ah)*((jkn.d.aucs-d.aucs)^2))
    d.sd <- sqrt(JKn.var)
    d.z <- d.aucs/d.sd

  } else {
    
    if(method == "subbootstrap"){
      RB.boot.w <- stats::weights(survey::as.svrepdesign(design = design, type = method, replicates=boot.B),"analysis")
      RB.aucw.b.1 <- as.vector(apply(
        RB.boot.w, 2,
        function(x) wauc(
          response.var = response.var,
          phat.var = phat.var.1,
          weights.var = x,
          tag.event = tag.event,
          tag.nonevent = tag.nonevent
        )$AUCw[1]
      ))
      RB.aucw.b.2 <- as.vector(apply(
        RB.boot.w, 2,
        function(x) wauc(
          response.var = response.var,
          phat.var = phat.var.2,
          weights.var = x,
          tag.event = tag.event,
          tag.nonevent = tag.nonevent
        )$AUCw[1]
      ))
      
      RB.aucw.Db <- RB.aucw.b.1-RB.aucw.b.2
      d.sd <- sd(RB.aucw.Db)
      d.z <- (wauc.1-wauc.2)/d.sd

    } else {
      stop("One of the following methods must be selected: method = 'JKn' or method = 'subbootstrap'.")
    }
    
  }
  
  if (alternative == "two.sided") {
    pval <- 2*pnorm(abs(d.z), lower.tail = FALSE)
  } else {
    if (alternative == "greater") {
      pval <- pnorm(d.z, lower.tail = FALSE)
    } else {
      if (alternative == "less") {
        pval <- pnorm(d.z, lower.tail = TRUE)
      } else {
        stop("One of the following options must be selected for the alternative hypothesis: 'two.sided', 'greater' or 'less'")
      }
    } 
  }  
  
  obj <- list(wauc.1 = wauc.1,
              wauc.2 = wauc.2,
              sd = d.sd,
              z = d.z,
              pvalue = pval)
  
  return(obj)
  
}

#' Estimation of the ROC curve of logistic regression models with complex survey data
#'
#' Calculate the ROC curve of a logistic regression model
#' considering sampling weights with complex survey data
#' @param response.var A character string with the name of the column indicating the response variable in the data set
#' or a vector (either numeric or character string) with information of the response variable for all the units.
#' @param phat.var A character string with the name of the column indicating the estimated probabilities in the data set
#' or a numeric vector containing estimated probabilities for all the units.
#' @param weights.var A character string indicating the name of the column with sampling weights or
#' a numeric vector containing information of the sampling weights.
#' It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
#' For unweighted estimates, set all the sampling weight values to 1.
#' @param tag.event A character string indicating the label used to indicate the event of interest in \code{response.var}.
#' The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.
#' @param tag.nonevent A character string indicating the label used for non-event in \code{response.var}.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param data A data frame which, at least, must incorporate information on the columns
#' \code{response.var}, \code{phat.var} and \code{weights.var}.
#' If \code{data=NULL}, then specific numerical vectors must be included in
#' \code{response.var}, \code{phat.var} and \code{weights.var},
#' or the sampling design should be indicated in the argument \code{design}.
#' @param design An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data.
#' If \code{design = NULL}, information on the data set (argument \code{data})
#' and/or sampling weights (argument \code{weights.var}) must be included.
#' @param cutoff.method A character string indicating the method to be used to select the optimal cut-off point.
#' If `cutoff.method = NULL`, then no optimal cut-off point is calculated.
#' If an optimal cut-off point is to be calculated, one of the following methods needs to be selected:
#' `Youden`, `MaxProdSpSe`, `ROC01`, `MaxEfficiency`.
#'
#' @return The output object of this function is a list of class `wroc`, which contains information about the weighted ROC curve of a logistic regression model and some of its components. In particular, this list contains a total of 5 or 6 elements (depending on the selected arguments) with the following information:
#' - `wroc.curve`: this element is a list that contains three numerical vectors. Specifically,
#'   - `Sew.values`: a vector of all the different values for the weighted estimate of the sensitivity across all the possible cut-off points.
#'   - `Spw.values`: a vector of all the different values for the weighted estimate of the specificity across all the possible cut-off points.
#'   - `cutoffs`: this vector contains all the cut-off points that have been considered to estimate sensitivity and specificity parameters.
#' - `wauc`: a numeric value indicating the area under the weighted estimate of the ROC curve.
#' - `optimal.cutoff`: if the argument `cutoff.method != NULL`, this object is a list containing the 4 elements described below:
#'   - `method`: character string indicating the method implemented to calculate the optimal cut-off point.
#'   - `cutoff.value`: the optimal cut-off point value.
#'   - `Spw`: the weighted estimate of the specificity for the optimal cut-off point value (indicated in `cutoff.value`).
#'   - `Sew`: the weighted estimate of the sensitivity for the optimal cut-off point value (indicated in `cutoff.value`).
#' - `tags`: a list containing two elements with the following information:
#'   - `tag.event`: a character string indicating the event of interest.
#'   - `tag.nonevent`: a character string indicating the non-event.
#' - `basics`: a list containing information of the following 4 elements:
#'   - `n.event`: number of units with the event of interest in the data set.
#'   - `n.nonevent`: number of units without the event of interest in the data set.
#'   - `hatN.event`: number of units with the event of interest represented in the population by all the event units in the data set, i.e., the sum of the sampling weights of the units with the event of interest in the data set.
#'   - `hatN.nonevent`: a numeric value indicating the number of non-event units in the population represented by means of the non-event units in the data set, i.e., the sum of the sampling weights of the non-event units in the data set.
#' - `call`: an object saving the information about the way in which the function has been run.
#'
#' @details
#' \eqn{S} indicate a sample of \eqn{n} observations of the vector of random variables \eqn{(Y,\pmb X)}, and \eqn{\forall i=1,\ldots,n,} \eqn{y_i} indicate the \eqn{i^{th}} observation of the response variable \eqn{Y},
#' and \eqn{\pmb x_i} the observations of the vector covariates \eqn{\pmb X}. Let \eqn{w_i} indicate the sampling weight corresponding to the unit \eqn{i} and \eqn{\hat p_i} the estimated probability of event.
#' Let \eqn{S_0} and \eqn{S_1} be subsamples of \eqn{S}, formed by the units without the event of interest (\eqn{y_i=0}) and with the event of interest (\eqn{y_i=1}), respectively.
#' Then, the ROC curve is estimated as follows:
#' \deqn{\widehat{ROC}_w(\cdot)=\{(1-\widehat{Sp}_w(c),\widehat{Se}_w(c)),\:c\in (-\infty, \infty)\}},
#' where, the sensitivity and specificity parameters for a given cut-off point \eqn{c} are estimated as follows:
#' \deqn{\widehat{Se}_w(c)=\dfrac{\sum_{i\in S_1}w_i\cdot I (\hat p_i\geq c)}{\sum_{i\in S_1}w_i}\:;\:\widehat{Sp}_w(c)=\dfrac{\sum_{i\in S_0}w_i\cdot I (\hat p_i<c)}{\sum_{i\in S_0}w_i}.}
#'
#' See Iparragirre et al (2023) for more information. More information of the rest of the elements is given in the documentation of the functions \code{wauc()} and \code{wocp()}.
#'
#'
#' @references Iparragirre, A., Barrio, I. and Arostegui, I. (2023).
#' Estimation of the ROC curve and the area under it with complex survey data.
#' *Stat* **12**(1), e635. (https://doi.org/10.1002/sta4.635)
#'
#' @export
#'
#'
#' @examples
#' data(example_data_wroc)
#'
#' mycurve <- wroc(response.var = "y", phat.var = "phat", weights.var = "weights",
#'                 data = example_data_wroc,
#'                 tag.event = 1, tag.nonevent = 0,
#'                 cutoff.method = "Youden")
#'
#' # Or equivalently
#' \donttest{
#' mycurve <- wroc(response.var = example_data_wroc$y,
#'                 phat.var = example_data_wroc$phat,
#'                 weights.var = example_data_wroc$weights,
#'                 tag.event = 1, tag.nonevent = 0,
#'                 cutoff.method = "Youden")
#'}
wroc <- function(response.var, phat.var, weights.var = NULL,
                 tag.event = NULL, tag.nonevent = NULL,
                 data = NULL, design = NULL,
                 cutoff.method = NULL){

  if(inherits(response.var, "character")){
    response.var <- data[,response.var]
  }

  if(length(table(response.var))!=2){stop("Response variable must have two classes.")}

  if(inherits(phat.var, "character")){
    phat.var <- data[,phat.var]
  }

  if(inherits(weights.var, "character")){
    weights.var <- data[,weights.var]
  }

  if(!is.null(design)){
    data <- get(design$call$data)
    weights <- as.character(design$call$weights[2])
    weights.var <- data[,weights]
  }

  if(length(response.var) != length(phat.var) || length(response.var) != length(weights.var)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }

  # Missing tags:
  if(is.null(tag.nonevent)){
    if(is.null(tag.event)){
      tag.nonevent <- names(table(response.var)[which.max(table(response.var))])
      tag.event <- names(table(response.var)[which.min(table(response.var))])
    } else {
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.event)
      tag.nonevent <- tags[-tag.other]
    }
  } else {
    if(is.null(tag.event)){
      tags <- names(table(response.var))
      tag.other <- which(tags == tag.nonevent)
      tag.event <- tags[-tag.other]
    }
  }


  all.phat <- sort(unique(phat.var))

  lower <- all.phat[1:(length(all.phat)-1)]
  upper <- all.phat[2:length(all.phat)]
  cutoffs <- rev(c(0, (lower+upper)/2, 1))

  # ROCw curve:
  sew.v <- sapply(lapply(cutoffs, wse, response.var = response.var, tag.event = tag.event, phat.var = phat.var, weights.var = weights.var), "[[", 1)
  spw.v <- sapply(lapply(cutoffs, wsp, response.var = response.var, tag.nonevent = tag.nonevent, phat.var = phat.var, weights.var = weights.var), "[[", 1)

  res <- list()
  res$wroc.curve <- list(Sew.values = sew.v, Spw.values = spw.v, cutoffs = cutoffs)

  # Area under the ROCw curve:
  #auc <- wauc(response.var, phat.var, weights.var, tag.event, tag.nonevent, data)
  lc <- length(cutoffs)
  auc <- sum(spw.v[-lc]*sew.v[-1]-spw.v[-1]*sew.v[-lc])/2
  res$wauc <- auc

  # Cut-off point:
  if(!is.null(cutoff.method)){
    if(any(!(cutoff.method %in% c("MaxProdSpSe","ROC01","Youden","MaxEfficiency")))) {
      stop ("You have entered an invalid method.", call. = FALSE)
    }
    ocp <- wocp(response.var = response.var, phat.var = phat.var,
                weights.var=weights.var,
                tag.nonevent=tag.nonevent,
                method=cutoff.method)
    res$optimal.cutoff$method <- cutoff.method
    res$optimal.cutoff$cutoff.value <- ocp[["optimal.cutoff"]][["optimal"]][["cutoff"]]
    res$optimal.cutoff$Spw <- ocp[["optimal.cutoff"]][["optimal"]][["Spw"]]
    res$optimal.cutoff$Sew <- ocp[["optimal.cutoff"]][["optimal"]][["Sew"]]
  }

  # Output object:
  res$tags <- list()
  res$tags$tag.event <- as.character(tag.event)
  res$tags$tag.nonevent <- as.character(tag.nonevent)
  res$basics <- list()
  res$basics$n.event <- length(which(response.var==tag.event))
  res$basics$n.nonevent <- length(which(response.var==tag.nonevent))
  res$basics$hatN.event <- sum(weights.var[which(response.var==tag.event)])
  res$basics$hatN.nonevent <- sum(weights.var[which(response.var==tag.nonevent)])
  res$call <- match.call()

  class(res) <- "wroc"

  return(res)

}

#' Estimation of the specificity with complex survey data
#'
#' Estimate the specificity parameter for a given cut-off point
#' considering sampling weights with complex survey data.
#' @param response.var A character string with the name of the column indicating the response variable in the data set
#' or a vector (either numeric or character string) with information of the response variable for all the units.
#' @param phat.var A character string with the name of the column indicating the estimated probabilities in the data set
#' or a numeric vector containing estimated probabilities for all the units.
#' @param weights.var A character string indicating the name of the column with sampling weights or
#' a numeric vector containing information of the sampling weights.
#' It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
#' For unweighted estimates, set all the sampling weight values to 1.
#' @param tag.nonevent A character string indicating the label used for non-event in \code{response.var}.
#' The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.
#' @param cutoff.value A numeric value indicating the cut-off point to be used.
#' No default value is set for this argument, and a numeric value must be indicated necessarily.
#' @param data A data frame which, at least, must incorporate information on the columns
#' \code{response.var}, \code{phat.var} and \code{weights.var}.
#' If \code{data=NULL}, then specific numerical vectors must be included in
#' \code{response.var}, \code{phat.var} and \code{weights.var},
#' or the sampling design should be indicated in the argument \code{design}.
#' @param design An object of class \code{survey.design} generated by
#' \code{survey::svydesign} indicating the complex sampling design of the data.
#' If \code{design = NULL}, information on the data set (argument \code{data})
#' and/or sampling weights (argument \code{weights.var}) must be included.
#'
#' @return The output of this function is a list of 4 elements containing the following information:
#' - `Spw`: a numeric value indicating the weighted estimate of the specificity parameter.
#' - `tags`: a list containing one element with the following information:
#'    - `tag.nonevent`: a character string indicating the label used for non-events.
#' - `basics`: a list containing information of the following 6 elements:
#'    - `n`: a numeric value indicating the number of units in the data set.
#'    - `n.nonevent`: a numeric value indicating the number of units in the data set without the event of interest.
#'    - `n.nonevent.class`: a numeric value indicating the number of units in the data set without the event of interest that are correctly classified as non-events based on the selected cut-off point.
#'    - `hatN`: a numeric value indicating the number of units in the population that are represented by means of the units in the data set, i.e., the sum of the sampling weights of all the units in the data set.
#'    - `hatN.nonevent`: a numeric value indicating the number of non-event units in the population represented by means of the non-event units in the data set, i.e., the sum of the sampling weights of the non-event units in the data set.
#'    - `hatN.nonevent.class`: number of non-event units represented in the population by the non-event units in the data set that have been correctly classified as non-events based on the selected cut-off point, i.e., the sum of the sampling weights of the correctly classified non-event units in the data set.
#' - `call`: an object saving the information about the way in which the function has been run.
#'
#' @details
#' Let \eqn{S} indicate a sample of \eqn{n} observations of the vector of random variables \eqn{(Y,\pmb X)}, and \eqn{\forall i=1,\ldots,n,} \eqn{y_i} indicate the \eqn{i^{th}} observation of the response variable \eqn{Y},
#' and \eqn{\pmb x_i} the observations of the vector covariates \eqn{\pmb X}. Let \eqn{w_i} indicate the sampling weight corresponding to the unit \eqn{i} and \eqn{\hat p_i} the estimated probability of event.
#' Let \eqn{S_0} and \eqn{S_1} be subsamples of \eqn{S}, formed by the units without the event of interest (\eqn{y_i=0}) and with the event of interest (\eqn{y_i=1}), respectively.
#' Then, the specificity parameter for a given cut-off point \eqn{c} is estimated as follows:
#' \deqn{\widehat{Sp}_w(c)=\dfrac{\sum_{i\in S_0}w_i\cdot I (\hat p_i<c)}{\sum_{i\in S_0}w_i}.}
#' See Iparragirre et al. (2022) and Iparragirre et al. (2023) for more details.
#'
#' @references Iparragirre, A., Barrio, I., Aramendi, J. and Arostegui, I. (2022).
#' Estimation of cut-off points under complex-sampling design data.
#' *SORT-Statistics and Operations Research Transactions* **46**(1), 137--158. (https://doi.org/10.2436/20.8080.02.121)
#' @references Iparragirre, A., Barrio, I. and Arostegui, I. (2023).
#' Estimation of the ROC curve and the area under it with complex survey data.
#' *Stat* **12**(1), e635. (https://doi.org/10.1002/sta4.635)
#'
#' @export
#' @examples
#' data(example_data_wroc)
#'
#' sp.obj <- wsp(response.var = "y",
#'               phat.var = "phat",
#'               weights.var = "weights",
#'               tag.nonevent = 0,
#'               cutoff.value = 0.5,
#'               data = example_data_wroc)
#'
#' # Or equivalently
#' sp.obj <- wsp(response.var = example_data_wroc$y,
#'               phat.var = example_data_wroc$phat,
#'               weights.var = example_data_wroc$weights,
#'               tag.nonevent = 0,
#'               cutoff.value = 0.5)
#' sp.obj

wsp <- function(response.var, phat.var, weights.var = NULL, tag.nonevent = NULL, cutoff.value, data = NULL, design = NULL){

  if(missing(cutoff.value) || !is.numeric(cutoff.value)){stop("Please, indicate a numeric value in the 'cutoff.value' argument to be used as cut-off point.")}

  if(inherits(response.var, "character")){
    response.var <- data[,response.var]
  }

  if(length(table(response.var))!=2){stop("Response variable must have two classes.")}

  if(inherits(phat.var, "character")){
    phat.var <- data[,phat.var]
  }

  if(inherits(weights.var, "character")){
    weights.var <- data[,weights.var]
  }

  if(!is.null(design)){
    data <- get(design$call$data)
    weights <- as.character(design$call$weights[2])
    weights.var <- data[,weights]
  }

  if(length(response.var) != length(phat.var) || length(response.var) != length(weights.var)){
    stop("Vectors indicating the responses, predicted probabilities and sampling weights must be the same length.")
  }

  if(is.null(tag.nonevent)){
    tag.nonevent <- names(table(response.var)[which.max(table(response.var))])
  }

  sum.weight <- sum(weights.var[which(response.var == tag.nonevent)])
  yhat.nonevent.w <- sum(weights.var[which(response.var == tag.nonevent & phat.var < cutoff.value)])

  spw.hat <- yhat.nonevent.w/sum.weight

  spw.obj <- list()

  spw.obj$Spw <- spw.hat
  spw.obj$tags <- list(tag.nonevent = as.character(tag.nonevent))
  spw.obj$basics <- list(n = length(response.var),
                         n.nonevent = length(which(response.var == tag.nonevent)),
                         n.nonevent.class = length(which(response.var == tag.nonevent & phat.var < cutoff.value)),
                         hatN = sum(weights.var),
                         hatN.nonevent = sum.weight,
                         hatN.nonevent.class = yhat.nonevent.w)
  spw.obj$call <- match.call()

  return(spw.obj)

}

#' Simulated data
#'
#' This dataset has been simulated in order to provide the users with an example dataset.
#'
#' @format ## `example_data_wroc`
#' A data frame with 740 rows and 3 columns:
#' \describe{
#'   \item{y}{Response variable}
#'   \item{phat}{Predicted probabilities}
#'   \item{weights}{Sampling weights}
#'   ...
#' }
"example_data_wroc"

optimize.criterion <- function(method = c("Youden", "MaxProdSpSe", "ROC01", "MaxEfficiency"),
                               criterion.v, se.v, sp.v, cutoffs){

  if(method %in% c("Youden", "MaxProdSpSe", "MaxEfficiency")){
    c <- cutoffs[which(round(criterion.v,10) == round(max(criterion.v,na.rm=TRUE),10))]
  } else {
    c <- cutoffs[which(round(criterion.v,10) == round(min(criterion.v,na.rm=TRUE),10))]
  }


  position <- which(cutoffs %in% c)

  optimal.se <- se.v[position]
  optimal.sp <- sp.v[position]
  optimal.cutoff <- c
  optimal.criterion <- unique(round(criterion.v[position], 10))

  res <- list(optimal = list(cutoff = optimal.cutoff,
                             Sew = optimal.se,
                             Spw = optimal.sp,
                             criterion = optimal.criterion),
              all = list(cutoff = cutoffs,
                         Sew = se.v,
                         Spw = sp.v,
                         criterion = criterion.v))
  return(res)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wse.R
\name{wse}
\alias{wse}
\title{Estimation of the sensitivity with complex survey data}
\usage{
wse(
  response.var,
  phat.var,
  weights.var = NULL,
  tag.event = NULL,
  cutoff.value,
  data = NULL,
  design = NULL
)
}
\arguments{
\item{response.var}{A character string with the name of the column indicating the response variable in the data set
or a vector (either numeric or character string) with information of the response variable for all the units.}

\item{phat.var}{A character string with the name of the column indicating the estimated probabilities in the data set
or a numeric vector containing estimated probabilities for all the units.}

\item{weights.var}{A character string indicating the name of the column with sampling weights or
a numeric vector containing information of the sampling weights.
It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
For unweighted estimates, set all the sampling weight values to 1.}

\item{tag.event}{A character string indicating the label used to indicate the event of interest in \code{response.var}.
The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.}

\item{cutoff.value}{A numeric value indicating the cut-off point to be used.
No default value is set for this argument, and a numeric value must be indicated necessarily.}

\item{data}{A data frame which, at least, must incorporate information on the columns
\code{response.var}, \code{phat.var} and \code{weights.var}.
If \code{data=NULL}, then specific numerical vectors must be included in
\code{response.var}, \code{phat.var} and \code{weights.var},
or the sampling design should be indicated in the argument \code{design}.}

\item{design}{An object of class \code{survey.design} generated by
\code{survey::svydesign} indicating the complex sampling design of the data.
If \code{design = NULL}, information on the data set (argument \code{data})
and/or sampling weights (argument \code{weights.var}) must be included.}
}
\value{
The output of this function is a list of 4 elements containing the following information:
\itemize{
\item \code{Sew}: a numeric value indicating the weighted estimate of the sensitivity parameter.
\item \code{tags}: list containing one element with the following information:
\itemize{
\item \code{tag.event}: a character string indicating the label used to indicate event of interest.
}
\item \code{basics}: a list containing information of the following 6 elements:
\itemize{
\item \code{n}: a numeric value indicating the number of units in the data set.
\item \code{n.event}: a numeric value indicating the number of units in the data set with the event of interest.
\item \code{n.event.class}: a numeric value indicating the number of units in the data set with the event of interest that are correctly classified as events based on the selected cut-off point.
\item \code{hatN}: number of units in the population, represented by all the units in the data set, i.e., the sum of the sampling weights of the units in the data set.
\item \code{hatN.event}: number of units with the event of interest represented in the population by all the event units in the data set, i.e., the sum of the sampling weights of the units with the event of interest in the data set.
\item \code{hatN.event.class}: number of event units represented in the population by the event units in the data set that have been correctly classified as events based on the selected cut-off point, i.e., the sum of the sampling weights of the correctly classified event units in the data set.
}
\item \code{call}: an object saving the information about the way in which the function has been run.
}
}
\description{
Estimate the sensitivity parameter for a given cut-off point
considering sampling weights with complex survey data.
}
\details{
Let \eqn{S} indicate a sample of \eqn{n} observations of the vector of random variables \eqn{(Y,\pmb X)}, and \eqn{\forall i=1,\ldots,n,} \eqn{y_i} indicate the \eqn{i^{th}} observation of the response variable \eqn{Y},
and \eqn{\pmb x_i} the observations of the vector covariates \eqn{\pmb X}. Let \eqn{w_i} indicate the sampling weight corresponding to the unit \eqn{i} and \eqn{\hat p_i} the estimated probability of event.
Let \eqn{S_0} and \eqn{S_1} be subsamples of \eqn{S}, formed by the units without the event of interest (\eqn{y_i=0}) and with the event of interest (\eqn{y_i=1}), respectively.
Then, the sensitivity parameter for a given cut-off point \eqn{c} is estimated as follows:
\deqn{\widehat{Se}_w(c)=\dfrac{\sum_{i\in S_1}w_i\cdot I (\hat p_i\geq c)}{\sum_{i\in S_1}w_i}.}
See Iparragirre et al. (2022) and Iparragirre et al. (2023) for more details.
}
\examples{
data(example_data_wroc)

se.obj <- wse(response.var = "y", phat.var = "phat", weights.var = "weights",
              tag.event = 1, cutoff.value = 0.5, data = example_data_wroc)

# Or equivalently
se.obj <- wse(response.var = example_data_wroc$y,
              phat.var = example_data_wroc$phat,
              weights.var = example_data_wroc$weights,
              tag.event = 1, cutoff.value = 0.5)
}
\references{
Iparragirre, A., Barrio, I., Aramendi, J. and Arostegui, I. (2022).
Estimation of cut-off points under complex-sampling design data.
\emph{SORT-Statistics and Operations Research Transactions} \strong{46}(1), 137--158. (https://doi.org/10.2436/20.8080.02.121)

Iparragirre, A., Barrio, I. and Arostegui, I. (2023).
Estimation of the ROC curve and the area under it with complex survey data.
\emph{Stat} \strong{12}(1), e635. (https://doi.org/10.1002/sta4.635)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ht.paired.R
\name{ht.paired}
\alias{ht.paired}
\title{Design-based hypothesis test for the comparison of two paired AUCs.}
\usage{
ht.paired(
  design,
  response.var,
  phat.var.1,
  phat.var.2,
  tag.event = NULL,
  tag.nonevent = NULL,
  alternative = c("two.sided", "greater", "less"),
  method = c("JKn", "subbootstrap"),
  boot.B = 1000
)
}
\arguments{
\item{design}{An object of class \code{survey.design} generated by
\code{survey::svydesign} indicating the complex sampling design of the data.}

\item{response.var}{A character string with the name of the column indicating the response variable in the data set.}

\item{phat.var.1}{A character string with the name of the column indicating the estimated probabilities obtained with model 1 in the data set.}

\item{phat.var.2}{A character string with the name of the column indicating the estimated probabilities obtained with model 2 in the data set.}

\item{tag.event}{A character string indicating the label used to indicate the event of interest in \code{response.var}.
The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.}

\item{tag.nonevent}{A character string indicating the label used for non-event in \code{response.var}.
The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.}

\item{alternative}{A character string specifying the alternative hypothesis: \code{'two.sided'}, \code{greater}, or \code{less}.
Default is \code{alternative = 'two.sided'}.}

\item{method}{A character string indicating the method implemented to estimate the variance:
\code{method = 'JKn'} or \code{method = 'subbootstrap'}. Default is \code{method = 'JKn'}.}

\item{boot.B}{A numeric value indicating the number of bootstrap resamples to be considered. Default is \code{boot.B=1000}. Only for \code{method = 'subbootstrap'}.}
}
\value{
The output object of this function is a list of 5 elements containing the following information:
\itemize{
\item \code{wauc.1}: a numeric value indicating the weighted point estimate of the AUC for model 1.
\item \code{wauc.2}: a numeric value indicating the weighted point estimate of the AUC for model 2.
\item \code{sd}: a numeric value indicating the square root of the estimated variance.
\item \code{z}: a numeric value indicating the value of the statistic.
\item \code{pvalue}: a numeric value indicating the p-value of the test.
}
}
\description{
Design-based hypothesis test for the comparison of two paired AUCs.
}
\examples{
\donttest{
mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights,
data = example_variables_wroc, nest=TRUE)
model.1 <- survey::svyglm(y ~ x1 + x2 + x3, design = mydesign, family = quasibinomial())
model.2 <- survey::svyglm(y ~ x4 + x5 + x6, design = mydesign, family = quasibinomial())
phat.1 <- predict(model.1, newdata = example_variables_wroc, type = "response")
phat.2 <- predict(model.2, newdata = example_variables_wroc, type = "response")
example_variables_wroc$phat.1 <- phat.1
example_variables_wroc$phat.2 <- phat.2
mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights,
data = example_variables_wroc, nest=TRUE)

# JKn-based hypothesis test:
ht.JKn <- ht.paired(design=mydesign, response.var = "y", phat.var.1 = "phat.1",
                    phat.var.2 = "phat.2", tag.event = 1, tag.nonevent = 0, 
                    method="JKn")
                    
# Bootstrap-based hypothesis test:
set.seed(123)
ht.boot <- ht.paired(design=mydesign, response.var = "y", phat.var.1 = "phat.1",
                     phat.var.2 = "phat.2", tag.event = 1, tag.nonevent = 0, 
                     method="subbootstrap")
}

}
\references{
Iparragirre, A., Lumley, T., & Barrio, I. (2026).
Design-Based Inference for the AUC with Complex Survey Data.
arXiv, 2603.28320.
(https://arxiv.org/abs/2603.28320)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corrected.wauc.R
\name{corrected.wauc}
\alias{corrected.wauc}
\title{Corrected estimate of the AUC based on replicate weights.}
\usage{
corrected.wauc(
  data = NULL,
  formula,
  tag.event = NULL,
  tag.nonevent = NULL,
  weights.var = NULL,
  strata.var = NULL,
  cluster.var = NULL,
  design = NULL,
  method = c("dCV", "JKn", "RB"),
  dCV.method = c("average", "pooling"),
  RB.method = c("subbootstrap", "bootstrap"),
  k = 10,
  R = 1,
  B = 200
)
}
\arguments{
\item{data}{A data frame which, at least, must incorporate information on the columns
\code{response.var}, \code{phat.var} and \code{weights.var}.
If \code{data=NULL}, the sampling design must be indicated in the argument \code{design}.}

\item{formula}{Formula of the model for which the AUC needs to be corrected.
The models are fitted by means of \code{survey::svyglm()} function.}

\item{tag.event}{A character string indicating the label used to indicate the event of interest in \code{response.var}.
The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.}

\item{tag.nonevent}{A character string indicating the label used for non-event in \code{response.var}.
The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.}

\item{weights.var}{A character string indicating the name of the column with sampling weights.
It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.}

\item{strata.var}{A character string indicating the name of the column with strata identifiers.
It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.}

\item{cluster.var}{A character string indicating the name of the column with cluster identifiers.
It could be \code{NULL} if the sampling design is indicated in the \code{design} argument
or the sampling design does not have considered clustering.}

\item{design}{An object of class \code{survey.design} generated by \code{survey::svydesign()}.
It could be \code{NULL} if information about \code{cluster.var}, \code{strata.var}, \code{weights.var} and \code{data} are given.}

\item{method}{A character string indicating the method to be applied to define replicate weights and correct the AUC.
Choose between: \code{JKn} (for the Jackknife Repeated Replication), \code{dCV} (for the design-based cross-validation),
\code{RB} (for the Rescaling Bootstrap).}

\item{dCV.method}{Only applies for the \code{dCV} method.
Choose between: \code{average} (for the averaging cross-validation) or \code{pooling} (for the pooling cross-validation).
Note: \code{pooling} is recommended over \code{average} (see, Iparragirre and Barrio (2024))}

\item{RB.method}{Only applies for the \code{RB} method.
Choose between: \code{subbootstrap} or \code{bootstrap} (see the documentation of \code{svyVarSel::replicate.weights()} for help).}

\item{k}{A numeric value indicating the number of folds to be defined.
Default is \code{k=10}. Only applies for the \code{dCV} method.}

\item{R}{A numeric value indicating the number of times the sample is partitioned. Default is \code{R=1}. Only applies for \code{dCV}, \code{split} or \code{extrapolation} methods.}

\item{B}{A numeric value indicating the number of bootstrap resamples. Default is \code{B=200}. Only applies for \code{bootstrap} and  \code{subbootstrap} methods.}
}
\value{
The output object of this function is a list of 5 elements containing the following information:
\itemize{
\item \code{corrected.AUCw}: the corrected estimate of the weighted AUC.
\item \code{correction.method}: the selected correction method.
\item \code{formula}: formula of the model that has been fitted.
\item \code{tags}: a list containing two elements with the following information:
\itemize{
\item \code{tag.event}: a character string indicating the event of interest.
\item \code{tag.nonevent}: a character string indicating the non-event.
}
\item \code{call}: an object saving the information about the way in which the function has been run.
}
}
\description{
Optimism correction of the AUC of logistic regression models
with complex survey data based on replicate weights methods.
}
\details{
See Iparragirre and Barrio (2024) for more information on the AUC correction methods and their performance.
}
\examples{
data(example_variables_wroc)
mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata,
                              weights = ~weights, nest = TRUE,
                              data = example_variables_wroc)
m <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = mydesign,
                    family = quasibinomial())
phat <- predict(m, newdata = example_variables_wroc, type = "response")
myaucw <- wauc(response.var = example_variables_wroc$y, phat.var = phat,
               weights.var = example_variables_wroc$weights)

# Correction of the AUCw:
set.seed(1)
res <- corrected.wauc(data = example_variables_wroc,
                      formula = y ~ x1 + x2 + x3 + x4 + x5 + x6,
                      tag.event = 1, tag.nonevent = 0,
                      weights.var = "weights", strata.var = "strata", cluster.var = "cluster",
                      method = "dCV", dCV.method = "pooling", k = 10, R = 20)
# Or equivalently:
\donttest{
set.seed(1)
res <- corrected.wauc(design = mydesign,
                      formula = y ~ x1 + x2 + x3 + x4 + x5 + x6,
                      tag.event = 1, tag.nonevent = 0,
                      method = "dCV", dCV.method = "pooling", k = 10, R = 20)
}

}
\references{
Iparragirre, A., Barrio, I. (2024).
Optimism Correction of the AUC with Complex Survey Data.
In: Einbeck, J., Maeng, H., Ogundimu, E., Perrakis, K. (eds)
Developments in Statistical Modelling. IWSM 2024. Contributions to Statistics.
Springer, Cham. https://doi.org/10.1007/978-3-031-65723-8_7
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wroc.plot.R
\name{wroc.plot}
\alias{wroc.plot}
\title{Estimation of the ROC curve of logistic regression models with complex survey data}
\usage{
wroc.plot(
  x,
  print.auc = TRUE,
  print.cutoff = FALSE,
  col.cutoff = "red",
  cex.text = 0.75,
  round.digits = 4
)
}
\arguments{
\item{x}{An object of class \code{wroc} obtained by means of the function \code{wroc()}.}

\item{print.auc}{A logical value. If \code{TRUE}, the value of the area under the ROCw curve (AUCw) is printed (default \code{print.auc = TRUE}).}

\item{print.cutoff}{A logical value. If \code{TRUE}, the value of the optimal cut-off point, and the corresponding weighted estimates of the sensitivity and specificity parameters are printed (default \code{print.cutoff = TRUE}).}

\item{col.cutoff}{A character string indicating the color in which the cut-off point is depicted. The default option is \code{col.cutoff = "red"}.}

\item{cex.text}{A numeric value indicating the size with which the information of the AUCw and optimal cut-off point is printed. The default option is \code{cex.text = 0.75}.}

\item{round.digits}{A numeric value indicating the number of digits that will be employed when printing the information about the AUCw and optimal cut-off point. The default option is \code{round.digits = 4}.}
}
\value{
a graph
}
\description{
Plot the ROC curve of a logistic regression model
considering sampling weights with complex survey data.
}
\details{
More information is given in the documentation of the \code{wroc()}, \code{wauc{}} and \code{wocp()} functions.
}
\examples{
data(example_data_wroc)

mycurve <- wroc(response.var = "y", phat.var = "phat", weights.var = "weights",
                data = example_data_wroc,
                tag.event = 1, tag.nonevent = 0,
                cutoff.method = "Youden")
wroc.plot(x = mycurve, print.auc = TRUE, print.cutoff = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wocp.R
\name{wocp}
\alias{wocp}
\title{Optimal cut-off points for complex survey data}
\usage{
wocp(
  response.var,
  phat.var,
  weights.var = NULL,
  tag.event = NULL,
  tag.nonevent = NULL,
  method = c("Youden", "MaxProdSpSe", "ROC01", "MaxEfficiency"),
  data = NULL,
  design = NULL
)
}
\arguments{
\item{response.var}{A character string with the name of the column indicating the response variable in the data set
or a vector (either numeric or character string) with information of the response variable for all the units.}

\item{phat.var}{A character string with the name of the column indicating the estimated probabilities in the data set
or a numeric vector containing estimated probabilities for all the units.}

\item{weights.var}{A character string indicating the name of the column with sampling weights or
a numeric vector containing information of the sampling weights.
It could be \code{NULL} if the sampling design is indicated in the \code{design} argument.
For unweighted estimates, set all the sampling weight values to 1.}

\item{tag.event}{A character string indicating the label used to indicate the event of interest in \code{response.var}.
The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.}

\item{tag.nonevent}{A character string indicating the label used for non-event in \code{response.var}.
The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.}

\item{method}{A character string indicating the method to be used to select the optimal cut-off point.
Choose one of the following methods (Lopez-Raton et al, 2014):
\code{MaxProdSpSe}, \code{ROC01}, \code{Youden}, \code{MaxEfficiency}.}

\item{data}{A data frame which, at least, must incorporate information on the columns
\code{response.var}, \code{phat.var} and \code{weights.var}.
If \code{data=NULL}, then specific numerical vectors must be included in
\code{response.var}, \code{phat.var} and \code{weights.var},
or the sampling design should be indicated in the argument \code{design}.}

\item{design}{An object of class \code{survey.design} generated by
\code{survey::svydesign} indicating the complex sampling design of the data.
If \code{design = NULL}, information on the data set (argument \code{data})
and/or sampling weights (argument \code{weights.var}) must be included.}
}
\value{
The output of this function is an object of class \code{wocp}. This object is a list that contains information about the following 4 elements:
\itemize{
\item \code{tags}: a list containing two elements with the following information:
\itemize{
\item \code{tag.event}: a character string indicating the event of interest.
\item \code{tag.nonevent}: a character string indicating the non-event.
}
\item \code{basics}: a list containing information of the following 4 elements:
\itemize{
\item \code{n.event}: number of units with the event of interest in the data set.
\item \code{n.nonevent}: number of units without the event of interest in the data set.
\item \code{hatN.event}: number of units with the event of interest represented in the population by all the event units in the data set, i.e., the sum of the sampling weights of the units with the event of interest in the data set.
\item \code{hatN.nonevent}: a numeric value indicating the number of non-event units in the population represented by means of the non-event units in the data set, i.e., the sum of the sampling weights of the non-event units in the data set.
}
\item \code{optimal.cutoff}: this object is a list of three elements containing the information described below:
\itemize{
\item \code{method}: a character string indicating the method implemented to select the optimal cut-off point.
\item \code{optimal}: a list containing information of the following four elements:
\itemize{
\item \code{cutoff}: a numeric vector indicating the optimal cut-off point(s) that optimize(s) the selected criterion.
\item \code{Sew}: a numeric vector indicating the estimated sensitivity parameter(s) corresponding to the optimal cut-off point(s) that optimize(s) the selected criterion.
\item \code{Spw}: a numeric vector indicating the estimated specificity parameter(s) corresponding to the optimal cut-off point(s) that optimize(s) the selected criterion.
\item \code{criterion}: a numeric value indicating the criterion value optimized by means of the selected optimal cut-off point(s).
}
\item \code{all}: a list containing information on the following four elements:
\itemize{
\item \code{cutoff}: a numeric vector indicating all the cut-off points considered.
\item \code{Sew}: a numeric vector indicating the estimated sensitivity parameters corresponding to all the considered cut-off points.
\item \code{Spw}: a numeric vector indicating the estimated sensitivity parameters corresponding to all the considered cut-off points.
\item \code{criterion}: a numeric vector indicating the values of the selected criterion corresponding to all the considered cut-off points.
}
}
\item \code{call}: an object saving the information about the way in which the function has been run.
}
}
\description{
Calculate optimal cut-off points for complex survey data (Iparragirre et al., 2022).
Some functions of the package OptimalCutpoints (Lopez-Raton et al, 2014) have been used and modified in order them to consider sampling weights.
}
\details{
Let \eqn{S} indicate a sample of \eqn{n} observations of the vector of random variables \eqn{(Y,\pmb X)}, and \eqn{\forall i=1,\ldots,n,} \eqn{y_i} indicate the \eqn{i^{th}} observation of the response variable \eqn{Y},
and \eqn{\pmb x_i} the observations of the vector covariates \eqn{\pmb X}. Let \eqn{w_i} indicate the sampling weight corresponding to the unit \eqn{i} and \eqn{\hat p_i} the estimated probability of event.
Let \eqn{S_0} and \eqn{S_1} be subsamples of \eqn{S}, formed by the units without the event of interest (\eqn{y_i=0}) and with the event of interest (\eqn{y_i=1}), respectively.
Then, the optimal cut-off points are obtained as follows:
\itemize{
\item \code{Youden}: \deqn{c_w^{\text{Youden}}=argmax_c\{\widehat{Se}_w(c) + \widehat{Sp}_w(c)-1\},}
\item \code{MaxProdSpSe}: \deqn{c_w^{\text{MaxProdSpSe}}=argmax_c\{\widehat{Se}_w(c) * \widehat{Sp}_w(c)\},}
\item \code{ROC01}: \deqn{c_w^{\text{ROC01}}=argmax_c\{(\widehat{Se}_w(c)-1)^2 + (\widehat{Sp}_w(c)-1)^2\},}
\item \code{MaxEfficiency}: \deqn{c_w^{\text{MaxEfficiency}}=argmax_c\{\hat p_{Y,w}\widehat{Se}_w(c) + (1-\hat p_{Y,w})\widehat{Sp}_w(c)\},}
}

where, the sensitivity and specificity parameters for a given cut-off point \eqn{c} are estimated as follows:
\deqn{\widehat{Se}_w(c)=\dfrac{\sum_{i\in S_1}w_i\cdot I (\hat p_i\geq c)}{\sum_{i\in S_1}w_i}\:;\:\widehat{Sp}_w(c)=\dfrac{\sum_{i\in S_0}w_i\cdot I (\hat p_i<c)}{\sum_{i\in S_0}w_i},}
and,
\deqn{\hat p_{Y,w}=\dfrac{\sum_{i\in S} w_i\cdot I(y_i=1)}{\sum_{i\in S} w_i}.}

See Iparragirre et al. (2022) and Lopez-Raton et al. (2014) for more information.
}
\examples{
data(example_data_wroc)

myocp <- wocp(response.var = "y", phat.var = "phat", weights.var = "weights",
              tag.event = 1, tag.nonevent = 0,
              method = "Youden",
              data = example_data_wroc)

# Or equivalently
myocp <- wocp(example_data_wroc$y, example_data_wroc$phat, example_data_wroc$weights,
              tag.event = 1, tag.nonevent = 0, method = "Youden")


}
\references{
Iparragirre, A., Barrio, I., Aramendi, J. and Arostegui, I. (2022). Estimation of cut-off points under complex-sampling design data. \emph{SORT-Statistics and Operations Research Transactions} \bold{46}(1), 137--158.

Lopez-Raton, M., Rodriguez-Alvarez, M.X, Cadarso-Suarez, C. and Gude-Sampedro, F. (2014). OptimalCutpoints: An R Package for Selecting Optimal Cutpoints in Diagnostic Tests. \emph{Journal of Statistical Software} \bold{61}(8), 1--36.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_variables_wroc.R
\docType{data}
\name{example_variables_wroc}
\alias{example_variables_wroc}
\title{Simulated data}
\format{
\subsection{\code{example_variables_wroc}}{

A data frame with 1720 rows and 10 columns:
\describe{
\item{y}{Response variable}
\item{x1,...,x6}{Covariates}
\item{strata}{Strata variable}
\item{cluster}{Cluster variable}
\item{weights}{Sampling weights}
...
}
}
}
\usage{
example_variables_wroc
}
\description{
This dataset has been simulated in order to provide the users with an example dataset.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ht.indep.R
\name{ht.indep}
\alias{ht.indep}
\title{Design-based hypothesis test for the comparison of two independent AUCs.}
\usage{
ht.indep(
  design.1,
  response.var.1,
  phat.var.1,
  design.2,
  response.var.2,
  phat.var.2,
  tag.event = NULL,
  tag.nonevent = NULL,
  alternative = c("two.sided", "greater", "less"),
  method = c("JKn", "subbootstrap"),
  boot.B = 1000
)
}
\arguments{
\item{design.1}{An object of class \code{survey.design} generated by
\code{survey::svydesign} indicating the complex sampling design of the data set 1.}

\item{response.var.1}{A character string with the name of the column indicating the response variable in the data set 1.}

\item{phat.var.1}{A character string with the name of the column indicating the estimated probabilities in the data set 1.}

\item{design.2}{An object of class \code{survey.design} generated by
\code{survey::svydesign} indicating the complex sampling design of the data set 2.}

\item{response.var.2}{A character string with the name of the column indicating the response variable in the data set 2.}

\item{phat.var.2}{A character string with the name of the column indicating the estimated probabilities in the data set 2.}

\item{tag.event}{A character string indicating the label used to indicate the event of interest.
The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.}

\item{tag.nonevent}{A character string indicating the label used for non-event.
The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.}

\item{alternative}{A character string specifying the alternative hypothesis: \code{'two.sided'}, \code{greater}, or \code{less}.
Default is \code{alternative = 'two.sided'}.}

\item{method}{A character string indicating the method implemented to estimate the variance:
\code{method = 'JKn'} or \code{method = 'subbootstrap'}. Default is \code{method = 'JKn'}.}

\item{boot.B}{A numeric value indicating the number of bootstrap resamples to be considered. Default is \code{boot.B=1000}. Only for \code{method = 'subbootstrap'}.}
}
\value{
The output object of this function is a list of 5 elements containing the following information:
\itemize{
\item \code{wauc.1}: a numeric value indicating the weighted point estimate of the AUC for model 1.
\item \code{wauc.2}: a numeric value indicating the weighted point estimate of the AUC for model 2.
\item \code{sd}: a numeric value indicating the square root of the estimated variance.
\item \code{z}: a numeric value indicating the value of the statistic.
\item \code{pvalue}: a numeric value indicating the p-value of the test.
}
}
\description{
Design-based hypothesis test for the comparison of two independent AUCs.
}
\examples{
\donttest{
des.1 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights, 
                           data = example_variables_wroc, nest=TRUE)
m.1 <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = des.1, family = quasibinomial())
example_variables_wroc$phat.1 <- predict(m.1, newdata = example_variables_wroc, type = "response")
des.1 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights,
                           data = example_variables_wroc, nest=TRUE)
des.2 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights, 
                           data = example_variables_wroc_2, nest=TRUE)
m.2 <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = des.2, family = quasibinomial())
example_variables_wroc$phat.2 <- predict(m.1, newdata = example_variables_wroc_2, type = "response")
des.2 <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights,
                           data = example_variables_wroc_2, nest=TRUE)
                           
# JKn-based hypothesis test:
jkn.ht <- ht.indep(design.1=des.1, response.var.1="y", phat.var.1 = "phat.1",
                   design.2=des.2, response.var.2="y", phat.var.2 = "phat.2",
                   tag.event=1, tag.nonevent=0, method="JKn")
                   
# Bootstrap-based hypothesis test:
boot.ht <- ht.indep(design.1=des.1, response.var.1="y", phat.var.1 = "phat.1",
                    design.2=des.2, response.var.2="y", phat.var.2 = "phat.2",
                    tag.event=1, tag.nonevent=0, method="subbootstrap")
}
                    
}
\references{
Iparragirre, A., Lumley, T., & Barrio, I. (2026).
Design-Based Inference for the AUC with Complex Survey Data.
arXiv, 2603.28320.
(https://arxiv.org/abs/2603.28320)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci.wauc.R
\name{ci.wauc}
\alias{ci.wauc}
\title{Design-based confidence intervals for the AUC.}
\usage{
ci.wauc(
  design,
  response.var,
  phat.var,
  tag.event = NULL,
  tag.nonevent = NULL,
  method = c("JKn", "subbootstrap"),
  boot.B = 1000,
  boot.ci = c("normal", "quantile"),
  conf.level = 0.95
)
}
\arguments{
\item{design}{An object of class \code{survey.design} generated by
\code{survey::svydesign} indicating the complex sampling design of the data.}

\item{response.var}{A character string with the name of the column indicating the response variable in the data set.}

\item{phat.var}{A character string with the name of the column indicating the estimated probabilities in the data set.}

\item{tag.event}{A character string indicating the label used to indicate the event of interest in \code{response.var}.
The default option is \code{tag.event = NULL}, which selects the class with the lowest number of units as event.}

\item{tag.nonevent}{A character string indicating the label used for non-event in \code{response.var}.
The default option is \code{tag.nonevent = NULL}, which selects the class with the greatest number of units as non-event.}

\item{method}{A character string indicating the method implemented to calculate the confidence interval:
\code{method = 'JKn'} or \code{method = 'subbootstrap'}. Default is \code{method = 'JKn'}.}

\item{boot.B}{A numeric value indicating the number of bootstrap resamples to be considered. Default is \code{boot.B=1000}. Only for \code{method = 'subbootstrap'}.}

\item{boot.ci}{Select \code{boot.ci = 'normal'} for standardized normal distribution-based confidence intervals
and \code{boot.ci = 'quantile'} for confidence intervals based on quantiles. Default is \code{boot.ci = 'normal'}. Only for \code{method = 'subbootstrap'}.}

\item{conf.level}{A numeric value between 0 and 1 indicating the confidence level. Default is \code{conf.level = 0.95}, resulting in a 95\% CI.}
}
\value{
The output object of this function is a list of 2 elements containing the following information:
\itemize{
\item \code{wauc}: a numeric value indicating the weighted point estimate of the AUC.
\item \code{ci}: a numeric vector indicating the lower and upper values of the confidence interval.
}
}
\description{
Design-based confidence intervals for the AUC.
}
\examples{
\donttest{
mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights=~weights, 
data = example_variables_wroc, nest=TRUE)
m <- survey::svyglm(y ~ x1 + x2 + x3 + x4 + x5 + x6, design = mydesign, family = quasibinomial())
phat <- predict(m, newdata = example_variables_wroc, type = "response")
example_variables_wroc$phat <- phat
mydesign <- survey::svydesign(ids = ~cluster, strata = ~strata, weights = ~weights, 
data = example_variables_wroc, nest=TRUE)

# JKn-based confidence interval:
ci.JKn <- ci.wauc(design=mydesign, response.var = "y", phat.var = "phat", 
                  tag.event = 1, tag.nonevent = 0, method="JKn")

# Bootstrap-based confidence interval:
set.seed(123)
ci.boot <- ci.wauc(design=mydesign, response.var = "y", phat.var = "phat", 
                   tag.event = 1, tag.nonevent = 0, method="subbootstrap")
}

}
\references{
Iparragirre, A., Lumley, T., & Barrio, I. (2026).
Design-Based Inference for the AUC with Complex Survey Data.
arXiv, 2603.28320.
(https://arxiv.org/abs/2603.28320)
}
